//This script is by Alex Turtle

WORLD_TEXT <- null
PARACHUTE_PROP <- null
TF_GLOW <- null

crate_type <- 0
maxweight <- 0

price_modifier <- 1.0

current_offset <- Vector(0, 0, 0)

interact_time <- 0.0
interact_price <- 0

forced <- false

function OnPostSpawn()
{
    EntFireByHandle(self, "AddOutPut", "disablereceiveshadows 1", -1, null, null)
    EntFireByHandle(self, "AddOutPut", "fademaxdist 4000", -1, null, null)
    EntFireByHandle(self, "AddOutPut", "fademindist 3500", -1, null, null)
    EntFireByHandle(self, "DisableShadow", "", -1, null, null)

    NetProps.SetPropBool(self, "m_bForcePurgeFixedupStrings", true)

    //AddThinkToEnt(self, "GlowThink")
}

function GlowThink()
{
    local frame = Time()

    local color = ((240 + (sin(frame) * 15).tointeger())) | ((240 + (sin(frame) * 15).tointeger()) << 8) | ((225 + (sin(frame) * 25).tointeger()) << 16) | (255 << 24)

    NetProps.SetPropInt(self, "m_clrRender", color)
}

function InitializeCrate()
{
    CalculateChances()
    SetCrate(PickRandomCrate(), false)
}

function CalculateChances()
{
    maxweight = 0
    for (local i = 0; CRATETYPE_POOL.len() > i; i++)
    {
        maxweight += CRATETYPE_POOL[i].base_chance
    }
}

function PickRandomCrate()
{
    local i = 0
    local chance = 0
    local rnd = RandomInt(0, maxweight)
    for (local i = 0; CRATETYPE_POOL.len() > i; i++)
    {
        chance += CRATETYPE_POOL[i].base_chance

        if (rnd < chance)
        {
            return i
        }
    }
    return i
}

function PickRandomLoot(lootArray)
{
    local loot_maxweight = 0
    for (local i = 0; lootArray.len() > i; i++)
    {
        loot_maxweight += lootArray[i].weight
    }
    
    local i = 0
    local chance = 0
    local rnd = RandomInt(0, loot_maxweight)

    //Calculate the loot chances modifier
    //local difference = rnd.tofloat() * loot_modifier - rnd.tofloat() 
    //rnd = rnd - ceil(difference)
    
    for (local i = 0; lootArray.len() > i; i++)
    {
        chance += lootArray[i].weight

        if (rnd < chance)
        {
            return lootArray[i]
        }
    }
    return lootArray[i]
}



function AirdropThink()
{
    if (PARACHUTE_PROP == null)
        CreateParachute()
    
    if (self.GetAbsVelocity().z >= 0.0)
    {
        self.SetGravity(100.0)
        self.KeyValueFromString("thinkfunction", "")
        EntFireByHandle(PARACHUTE_PROP, "SetDefaultAnimation", "retract_idle", 0.0, null, null)
        EntFireByHandle(PARACHUTE_PROP, "SetAnimation", "retract", 0.0, null, null)
        EntFireByHandle(self, "RunScriptCode", "KillParachuteEvent()", 0.5, null, null)
        EntFireByHandle(self, "RunScriptCode", "KillGlowEvent()", 5.5, null, null)
    }
    return 1.0
}

function KillParachuteEvent()
{
    PARACHUTE_PROP.Kill()
}

function KillGlowEvent()
{
    TF_GLOW.Kill()
}

function CreateParachute()
{
    local location = self.GetOrigin()
    local angle = self.GetAngles()
    PARACHUTE_PROP = SpawnEntityFromTable("prop_dynamic", 
    {
        // This is a table of keyvalues, which is the same way as keyvalues that are defined in Hammer
        // Key         Value
        model = "models/workshop/weapons/c_models/c_paratooper_pack/c_paratrooper_parachute.mdl",
        parentname = self.GetName(),
        DefaultAnim = "deploy_idle",
        origin     = Vector(location.x + 24, location.y + 2, location.z - (CRATETYPE_POOL[crate_type].offset.z * CRATETYPE_POOL[crate_type].scale) - 90),
        modelscale = 2,
        disableshadows = 1,
        disablereceiveshadows = 1,
    })
    EntFireByHandle(PARACHUTE_PROP, "SetParent", "!activator", 0.0, self, self)
    EntFireByHandle(PARACHUTE_PROP, "SetAnimation", "deploy", 0.0, null, null)

    TF_GLOW = SpawnEntityFromTable("tf_glow", 
    {
        // This is a table of keyvalues, which is the same way as keyvalues that are defined in Hammer
        // Key         Value
        target = self.GetName(),
        GlowColor = "255 255 255 255",
        StartDisabled = 0,
    })

}

function SetCrate(name, force = true)
{
    if (forced)
        return

    crate_type = name
    foreach (index, item in CRATETYPE_POOL)  {
        if (item.name == name)  {crate_type = index; break}}


    interact_time = CRATETYPE_POOL[crate_type].unbox_time
    interact_price = floor(CRATETYPE_POOL[crate_type].base_cost * price_modifier)
    self.SetModelSimple(CRATETYPE_POOL[crate_type].model)
    self.Teleport(true, self.GetLocalOrigin() + CRATETYPE_POOL[crate_type].offset - current_offset, false, self.GetAbsAngles(), false, Vector(0, 0, 0))
    self.SetModelScale(CRATETYPE_POOL[crate_type].scale, 0.0)
    self.SetSkin(CRATETYPE_POOL[crate_type].skin)
    current_offset = CRATETYPE_POOL[crate_type].offset

    //Workaround for models without collisions, smaller hitbox
    if (!CRATETYPE_POOL[crate_type].hasCollision)
    {
        local mins = self.GetBoundingMinsOriented()
        local maxs = self.GetBoundingMaxsOriented()

        self.KeyValueFromInt("solid", 2)
        self.SetSize(Vector(mins.x * 0.25, mins.y * 0.25, mins.z), Vector(maxs.x * 0.25, maxs.y * 0.25, maxs.z))
    }

    if (WORLD_TEXT)
    {
        WORLD_TEXT.Kill()
        WORLD_TEXT = null
    }

    if (interact_price > 0)
    {
        //local text = interact_price > 0? interact_price + "$":"!"

        local location = self.GetOrigin()
        WORLD_TEXT = SpawnEntityFromTable("point_worldtext", 
        {
        // This is a table of keyvalues, which is the same way as keyvalues that are defined in Hammer
        // Key         Value
            message      = interact_price + "$"
            color        = "255 255 0"
            textsize     = 24
            font         = 0
            orientation  = 1
            origin       = Vector(location.x, location.y, location.z + self.GetBoundingMaxsOriented().z + 16.0)
        })
        EntFireByHandle(WORLD_TEXT, "SetParent", "!activator", 0.0, self, self)
    }

    forced = force
}

function Interacting(player)
{
    if (player.GetCurrency() >= interact_price)
    {
        if (interact_price > 0)
        {
            local playerOrigin = player.EyePosition()
            DispatchParticleEffect("br_coins", Vector(playerOrigin.x, playerOrigin.y, playerOrigin.z - 26.0), player.EyeAngles().Forward())
        }
        return
    }
    else
    {
        return
    }

}

function OnInteractSuccess(player)
{
    if (!CratesEnabled)
        return

    EmitSoundOnClient(CRATETYPE_POOL[crate_type].sound, player)

    local origin = self.GetOrigin() - CRATETYPE_POOL[crate_type].offset

    local distributed_loot = []

    //Drop loot logic
    foreach(loot in CRATETYPE_POOL[crate_type].lootArray)
    {
        //Drops no matter what
        if (loot.lootPool < 0)
        {
            if (loot.BR_Function == null)
                continue
            
            this[loot.BR_Function].call(this, loot.parameter, self.GetOrigin() - CRATETYPE_POOL[crate_type].offset)
        }
        else if (distributed_loot.find(loot.lootPool) == null)
        {
            distributed_loot.append(loot.lootPool)
            local list = CRATETYPE_POOL[crate_type].lootArray.filter(function(index, value) {
            return (value.lootPool == loot.lootPool);
            });
            list.sort(@(a,b) b.weight <=> a.weight);

            local picked_loot = PickRandomLoot(list)

            if (picked_loot.BR_Function == null)
                continue

            this[picked_loot.BR_Function].call(this, picked_loot.parameter, self.GetOrigin() - CRATETYPE_POOL[crate_type].offset)
        }
    }

    if (WORLD_TEXT != null)
        WORLD_TEXT.Kill()

    self.Kill()
}

function DisplayInformation(player)
{
    if (!CratesEnabled)
        return "Crates are currently unavailable"

    return "Hold MOUSE3 or ACTION SLOT to unbox " + CRATETYPE_POOL[crate_type].name + "\n('H' by default)"
}

function DisplayInteracting(player, time_elapsed)
{
    if (!CratesEnabled)
        return "Crates are currently unavailable"

    return "Unboxing\n" + DisplayStylizedCounter(time_elapsed, interact_time, 0.5)
}

function DisplayCannotAfford(player)
{
    return "You cannot afford this crate"
}