//This script is by Alex Turtle

::BUTTONS <-
{
    IN_SCORE = 65536
    IN_ATTACK3 = 33554432
}

::IsValidPlayer <- function(player)
{
    return player != null && player.IsValid() && player.GetTeam() > 1
}

::IsAlivePlayer <- function(player)
{
    return IsValidPlayer(player) && NetProps.GetPropInt(player, "m_lifeState") == 0 && player.GetTeam() == 2
}

::FixMannpower <- function(entity)
{

    for (local i = 0; i < MANNPOWER_POOL.len(); i++) 
    {
        if  (MANNPOWER_POOL[i].modelpath == entity.GetModelName())
        {
            SpawnMannpower(i, entity.GetOrigin())
            return
        }
    }
}

::SpawnRandomWeapon <- function(rarity, location)
{
    local propDynamic = SpawnEntityFromTable("prop_dynamic", 
    {
        // This is a table of keyvalues, which is the same way as keyvalues that are defined in Hammer
        // Key         Value
        model = "models/weapons/c_models/c_shotgun/c_shotgun.mdl",
        vscripts   = "battleroyale/battleroyale_weapon.nut",
        thinkfunction = "BobbingThink",
        origin     = Vector(location.x, location.y, location.z + 64.0),
        solid       = 0,
        disableshadows = 1,
        StartDisabled  = 1,
        disablereceiveshadows = 1,
        fademaxdist = 4000,
        fademindist = 3500,
    })
    propDynamic.GetScriptScope().weapon <- CreateRandomBRWeapon(rarity)

}

::SpawnRandomAustraliumWeapon <- function(rarity, location)
{
    local propDynamic = SpawnEntityFromTable("prop_dynamic", 
    {
        // This is a table of keyvalues, which is the same way as keyvalues that are defined in Hammer
        // Key         Value
        model = "models/weapons/c_models/c_shotgun/c_shotgun.mdl",
        vscripts   = "battleroyale/battleroyale_weapon.nut",
        thinkfunction = "BobbingThink",
        origin     = Vector(location.x, location.y, location.z + 64.0),
        solid       = 0,
        disableshadows = 1,
        StartDisabled  = 1,
        disablereceiveshadows = 1,
        fademaxdist = 4000,
        fademindist = 3500,
    })
    propDynamic.GetScriptScope().weapon <- CreateRandomAustraliumBRWeapon(rarity)

}

::SpawnMoney <- function(amount, location)
{
    local currencyPack = SpawnEntityFromTable("item_armor", 
    {
        // This is a table of keyvalues, which is the same way as keyvalues that are defined in Hammer
        // Key         Value
        //powerup_model = model,
        origin     = Vector(location.x, location.y, location.z + 8.0),
        vscripts   = "battleroyale/battleroyale_cash.nut",
        AutoMaterialize = 0,
        StartDisabled  = 0,
        fademaxdist = 4000,
        fademindist = 3500,
    })
    currencyPack.GetScriptScope().value <- amount
    /*EntityOutputs.AddOutput(currencyPack, "OnCacheInteraction", "!self", "FireUser4", "", 0.0, -1)
    currencyPack.SetMoveType(Constants.EMoveType.MOVETYPE_FLYGRAVITY, Constants.EMoveCollide.MOVECOLLIDE_FLY_BOUNCE)
    local rnd1 = RandomInt(0, 10)
    local rnd2 = RandomInt(0, 10)
    currencyPack.ApplyAbsVelocityImpulse(Vector(rnd1, rnd2, 100))*/
}



::SpawnPickup <- function(pickup, location)
{
    local pickup = SpawnEntityFromTable(pickup, 
    {
        // This is a table of keyvalues, which is the same way as keyvalues that are defined in Hammer
        // Key         Value
        //powerup_model = model,
        origin     = Vector(location.x, location.y, location.z + 8.0),
        AutoMaterialize = 0,
        StartDisabled  = 0,
        fademaxdist = 4000,
        fademindist = 3500,
    })

    pickup.SetMoveType(Constants.EMoveType.MOVETYPE_FLYGRAVITY, Constants.EMoveCollide.MOVECOLLIDE_FLY_BOUNCE)
    local rnd1 = RandomInt(-100, 100)
    local rnd2 = RandomInt(-100, 100)
    pickup.ApplyAbsVelocityImpulse(Vector(rnd1, rnd2, 300))
}

::SpawnMannpower <- function(index, location)
{
    local mannpower = SpawnEntityFromTable("item_armor", 
    {
        // This is a table of keyvalues, which is the same way as keyvalues that are defined in Hammer
        // Key         Value
        powerup_model = MANNPOWER_POOL[index].modelpath,
        origin     = Vector(location.x, location.y, location.z + 16.0),
        vscripts   = "battleroyale/battleroyale_mannpower.nut",
        AutoMaterialize = 0,
        StartDisabled  = 0,
        fademaxdist = 4000,
        fademindist = 3500,
    })
    mannpower.GetScriptScope().index <- index
    mannpower.SetMoveType(Constants.EMoveType.MOVETYPE_FLYGRAVITY, Constants.EMoveCollide.MOVECOLLIDE_FLY_BOUNCE)
    local rnd1 = RandomInt(-50, 50)
    local rnd2 = RandomInt(-50, 50)
    mannpower.ApplyAbsVelocityImpulse(Vector(rnd1, rnd2, 250))
}

::VectorAngles <- function(forward)
{
	local yaw, pitch;
	if ( forward.y == 0.0 && forward.x == 0.0 )
	{
		yaw = 0.0;
		if (forward.z > 0.0)
			pitch = 270.0;
		else
			pitch = 90.0;
	}
	else
	{
		yaw = (atan2(forward.y, forward.x) * 180.0 / Constants.Math.Pi);
		if (yaw < 0.0)
			yaw += 360.0;
		pitch = (atan2(-forward.z, forward.Length2D()) * 180.0 / Constants.Math.Pi);
		if (pitch < 0.0)
			pitch += 360.0;
	}

	return QAngle(pitch, yaw, 0.0);
}

::CanBeBackstabbed <- function(attacker, victim)
{
    local ToTarget = victim.GetCenter() - attacker.GetCenter()
    ToTarget.z = 0.0

    ToTarget.Norm()

    local AttackerForward = attacker.EyeAngles().Forward()
    AttackerForward.z = 0
    AttackerForward.Norm()

    local VictimForward = victim.EyeAngles().Forward()
    VictimForward.z = 0
    VictimForward.Norm()


    local PosVsVictimViewDot = ToTarget.Dot(VictimForward)
    local PosVsAttackerViewDot = ToTarget.Dot(AttackerForward)
    local ViewAnglesDot = VictimForward.Dot(AttackerForward)

    return PosVsVictimViewDot > 0.0 && PosVsAttackerViewDot > 0.5 && ViewAnglesDot > -0.3
}

::arraySum <- function(arr)
{
    local sum = 0;
    foreach (element in arr) {
        sum += element;
    }
    return sum;
}

::GetActivePlayerCount <- function()
{
    local count = 0
    local ent = null
    while (ent = Entities.FindByClassname(ent, "player") )
    {
        if (IsValidPlayer(ent))
        {
            count++
        }
    }

    return count
}


::GetAlivePlayerCount <- function()
{
    local count = 0
    local ent = null
    while (ent = Entities.FindByClassname(ent, "player") )
    {
        if (IsAlivePlayer(ent))
        {
            count++
        }
    }

    return count
}

::GetCurrentPlayerCount <- function()
{
    local playerCount = 0
    local player = null
    while (player = Entities.FindByClassname(player, "player"))
    {
        playerCount++
    }
    return playerCount
}

::DisplayStylizedCounter <- function(time_elapsed, max_time, time_per_part)
{
    local text = ""
    local parts = max_time.tofloat() / time_per_part.tofloat()
    parts = ceil(parts)
    for (local i = 1 ; i < parts + 1 ; i++)
    {
        if (time_elapsed > time_per_part * i)
            text += "▰"
        else
            text += "▱"
    }

    return text
}


::LateRespawn <- function()
{
    activator.ForceRespawn()
}

::DisableLateRespawn <- function()
{
    EarlyDeathRespawn = false
}

::EnableRevives <- function()
{
    RevivesEnabled = true
}

::EnableCrates <- function()
{
    CratesEnabled = true
}

::ReturnPlayername <- function(player)
{
    return "\x07" + format("%02X%02X%02X", 186, 52, 53) + NetProps.GetPropString(player, "m_szNetname")
}

::FormatTextCash <- function(amount)
{
    return "\x07" + format("%02X%02X%02X", 122, 140, 92) + floor(amount) + "$"
}

::FormatTextWhite <- function(text)
{
    return "\x07" + format("%02X%02X%02X", 255, 255, 255) + text
}

::DEBUG_CountAllEntities <- function()
{
    local count = 0
    local ent = null;
    while ( ent = Entities.FindByClassname(ent, "*") )
    {
    count++
    }
    printl(count)
}