//Loot content - This is where the loot for each crate is defined
//You may add new loot in the list below - Each constructor is as follow:
//LOOT(lootPool, weight, BR_Function = null, parameter = null)
//lootPool - Loot part of the same lootPool number will compete to be chosen as the drop, 
//if you set -1 it will be a guaranteed drop
//weight: Chance of the loot within the pool to be chosen, total doesn't have to be 100
//BR_Function: if this loot gets chosen, which function to run?
//parameter: parameter for the BR_Function

//If you want to make a new crate with its own loot content, make a new array, feel free to copy the existing ones!
//Don't forget to add the array loot to the crate you're creating!

//Available functions for BR_Function:
//SpawnMoney - the parameter is the amount of money
//SpawnWeapon - the parameter is the rarity
//SpawnPickup - the parameter is the classname to spawn, i.e. item_ammopack_small
//SpawnMannpower - the parameter is the Mannpower to spawn

//Loot usually used by the most common crate, and also free!
CRATE_FREE_LOOT <-
[
    //Guaranteed loot
    LOOT(-1, 100, "SpawnMoney", 150)

    //Choose one between pool 1
    LOOT(1, 85, "SpawnRandomWeapon", RARITY.COMMON)
    LOOT(1, 15, "SpawnRandomWeapon", RARITY.UNCOMMON)

    //Choose one between pool 2
    LOOT(2, 10, "SpawnMoney", 50)
    LOOT(2, 20, "SpawnPickup", "item_healthkit_small")
    LOOT(2, 20, "SpawnPickup", "item_ammopack_small")
    LOOT(2, 50) //Chance to spawn nothing
]

//Loot for a crate you pay some money for
CRATE_RARE_LOOT <-
[
    //Guaranteed loot
    LOOT(-1,100, "SpawnPickup", "item_healthkit_medium")
    //Guaranteed loot
    LOOT(-1,100, "SpawnPickup", "item_ammopack_medium")

    //Choose one between pool 1
    LOOT(1,80, "SpawnRandomWeapon", RARITY.RARE)
    LOOT(1,20, "SpawnRandomWeapon", RARITY.LEGENDARY)
]

CRATE_BIG_LOOT <-
[
    //Guaranteed loot
    LOOT(-1,100, "SpawnPickup", "item_healthkit_small")
    //Guaranteed loot
    LOOT(-1,100, "SpawnPickup", "item_ammopack_small")

    //Choose one between pool 1
    LOOT(1,85, "SpawnRandomWeapon", RARITY.UNCOMMON)
    LOOT(1,15, "SpawnRandomWeapon", RARITY.RARE)
]

//Loot for a bag full of cash
CRATE_MONEY_LOOT <-
[
    //Guaranteed loot
    LOOT(-1, 100, "SpawnMoney", 150)

    //Choose one between pool 1
    LOOT(1, 50, "SpawnMoney", 100) 
    LOOT(1, 50, "SpawnMoney", 150)

    //Choose one between pool 2
    LOOT(2, 66, "SpawnMoney", 150)
    LOOT(2, 33, "SpawnMoney", 200)
]

//Loot for a rare expensive legendary crate
CRATE_LEGENDARY_LOOT <-
[
    //Guaranteed loot
    LOOT(-1,100, "SpawnPickup", "item_healthkit_medium")
    //Guaranteed loot
    LOOT(-1,100, "SpawnPickup", "item_ammopack_medium")

    //Choose one between pool 1
    LOOT(1,80, "SpawnRandomWeapon", RARITY.LEGENDARY)
    LOOT(1,20, "SpawnRandomWeapon", RARITY.EXPERIMENTAL)

    //Choose one between pool 2
    LOOT(2,50, "SpawnPickup", "item_healthkit_medium")
    LOOT(2,50, "SpawnPickup", "item_ammopack_medium")

    //Choose one between pool 3
    LOOT(3, 1, "SpawnMannpower", RUNETYPE.RUNE_STRENGTH)
    LOOT(3, 2, "SpawnMannpower", RUNETYPE.RUNE_PRECISION)
    LOOT(3, 2, "SpawnMannpower", RUNETYPE.RUNE_RESIST)
    LOOT(3, 2, "SpawnMannpower", RUNETYPE.RUNE_HASTE)
    LOOT(3, 70) //Chance to spawn nothing
]

//Loot array that spawns a Mannpower, please only use these 4 as they are the only balanced ones (and functioning)
CRATE_MANNPOWER_LOOT <-
[
    //Guaranteed loot
    LOOT(-1,100, "SpawnPickup", "item_healthkit_medium")
    //Guaranteed loot
    LOOT(-1,100, "SpawnPickup", "item_ammopack_medium")

    //Choose one between pool 1
    LOOT(1, 1, "SpawnMannpower", RUNETYPE.RUNE_STRENGTH)
    LOOT(1, 2, "SpawnMannpower", RUNETYPE.RUNE_PRECISION)
    LOOT(1, 2, "SpawnMannpower", RUNETYPE.RUNE_RESIST)
    LOOT(1, 2, "SpawnMannpower", RUNETYPE.RUNE_HASTE)
]

//Loot that mostly contains health and ammo, and some money
CRATE_SUPPLIES_LOOT <-
[
    //Guaranteed loot
    LOOT(-1, 100, "SpawnMoney", 100)

    //Choose one between pool 1
    LOOT(1,60, "SpawnPickup", "item_healthkit_small")
    LOOT(1,40, "SpawnPickup", "item_healthkit_medium")
    LOOT(1,20, "SpawnPickup", "item_healthkit_full")
    
    //Choose one between pool 2
    LOOT(2,60, "SpawnPickup", "item_ammopack_small")
    LOOT(2,40, "SpawnPickup", "item_ammopack_medium")
    LOOT(2,20, "SpawnPickup", "item_ammopack_full")

    //Choose one between pool 3
    LOOT(3,20, "SpawnPickup", "item_healthkit_small")
    LOOT(3,20, "SpawnPickup", "item_ammopack_small")
    LOOT(3,10, "SpawnMoney", 100)

    //Choose one between pool 4
    LOOT(4,30) //Chance to spawn nothing
    LOOT(4,20, "SpawnPickup", "item_healthkit_small")
    LOOT(4,20, "SpawnMoney", 50)
    LOOT(4,10, "SpawnMoney", 100)
]

//Loot that spawns an Australium weapon, a "sidegrade" in this gamemode
CRATE_AUSTRALIUM_LOOT <-
[
    //Guaranteed loot
    LOOT(-1,100, "SpawnPickup", "item_healthkit_medium")
    //Guaranteed loot
    LOOT(-1,100, "SpawnPickup", "item_ammopack_medium")

    //Choose one between pool 1
    LOOT(1,55, "SpawnRandomWeapon", RARITY.AUSTRALIUM_UNCOMMON)
    LOOT(1,30, "SpawnRandomWeapon", RARITY.AUSTRALIUM_RARE)
    LOOT(1,15, "SpawnRandomWeapon", RARITY.AUSTRALIUM_LEGENDARY)
]

//Example for custom loot, used in br_tikal, similar to the australium crate but spawns lesser quality weapons
CRATE_GOLD_IDOL_LOOT <-
[
    //Choose one between pool 1
    LOOT(1,50, "SpawnRandomWeapon", RARITY.AUSTRALIUM_UNCOMMON)
    LOOT(1,25, "SpawnRandomWeapon", RARITY.AUSTRALIUM_RARE)
    LOOT(1,10, "SpawnRandomWeapon", RARITY.AUSTRALIUM_LEGENDARY)
]