//--> To execute this script, make sure to place it in directory `/tf/scripts/vscripts/`
//--> Then to execute this script in a map, make an entity with "vscripts" keyvalue, and place `give_tf_weapon/_master`
//--> Alternatively, you can test it in any map through console by typing `script_execute give_tf_weapon/_master`

/*vScript "give_tf_weapon" (originally giveweapon.nut) framework for Team Fortress 2, by Yaki
Special thanks ficool2 for finding the netprops to make the first weapon giving function, AddWeapon().
Special thanks Mr.Burguers (TF2Maps) for being a wealth of knowledge, teaching me how to use vscripts.
Special thanks devs of Super Zombie Fortress + SCP Secret Fortress for releasing their source code. (taught me what netprops were needed for custom viewmodels)
Special thanks for those who sent bug reports (Lizard of Oz, SpookyToad), and the rest!

Notes to Users
 - Set CVAR_GTFW_* for easy tweaking of script.
 - When using GiveWeapon, weapon strings for unlocks don't need "The " in them. (i.e. "The Sandvich" is invalid, but "Sandvich" is acceptable)
 --> Also accepts handles, as well as item index values (from items_game.txt)
 - GiveWeapon also returns the weapon as a handle. So you can add attributes to the weapon!
 - Given/Custom weapons don't delete themselves on respawn. 
 ->Add function `CTFPlayer.GTFW_Cleanup` to your `post_inventory_application` event to delete any unused weapons/viewmodels attached to the player.
 --> Please place at the beginning!
 --> Needs a player handle to clear weapons from.
	
Clarification
		-> CTFPlayer = handle ID for your player's entity (aka player's serial number), called hPlayer all throughout examples.
	The following list are acceptable parameters for searching for weapons:
		-> Handles (aka the weapon ID's serial number)
		-> Classnames of weapons (i.e. "tf_weapon_wrench")
		-> items_game.txt weapon IDs are 0 and above (i.e. 7) (7 is the stock wrench)
		-> Slots, using TF_WEAPONSLOTS with slot type. 
			-> TF_WEAPONSLOTS.PRIMARY, for example, gets all slots in the primary slot.
			Also acceptable:
			TF_WEAPONSLOTS.SECONDARY
			TF_WEAPONSLOTS.MELEE
			TF_WEAPONSLOTS.SLOT3 = Engineer's "Build PDA", Spy's "Disguise Kit PDA"
			TF_WEAPONSLOTS.SLOT4 = Engineer's "Destroy PDA", Spy's watches
			TF_WEAPONSLOTS.SLOT5 = Engineer toolbox, sapper (both stock are "tf_weapon_builder", sapper unlocks are "tf_weapon_sapper")
			TF_WEAPONSLOTS.SLOT6 = spellbook, grappling hook. (PASSTIME gun?)


PUBLIC OPERATIONS
 Made to make giving weapons, easy! Also supports custom weapons!
	CheckItems()
		Checks your current equipment, with item IDs. Spits out a list in chat with active weapon on top.
	CTFPlayer.GiveWeapon(weapon)
		Gives you a weapon. Any weapon, as long as the entity and ID match from items_game.txt.
		Returns `weapon` as handle.
	CTFPlayer.DeleteWeapon(weapon)
		Deletes a weapon from the player.
		Returns true if weapon was deleted, else false.
	CTFPlayer.DisableWeapon(weapon)
		Removes all ammo from a weapon, and sets ammo clip + reserve for that weapon to 0.
		Returns `weapon` as handle.
	CTFPlayer.EnableWeapon(weapon)
		Resets ammo capacity, allowing player to grab ammo again.
		Returns `weapon` as handle.
	CTFPlayer.ReturnWeapon(weapon)
		Searches weapon on player and returns as a handle.
	CTFPlayer.ReturnWeaponTable(weapon)
		Find item on player by string, item ID, classname, slot, or handle. Else null.
		Returns a table with all properties found in `class TF_WEAPONS_BASE` (in `tables.nut`)
	CTFPlayer.ReturnWeaponBySlot
		Finds an equip by slot. 0 = Primary, 1 = Secondary, etc. Up to 6. Returns null if failed.
	RegisterCustomWeapon(item_name, baseitem, use_original_stats, stats_function, custom_arms_model, custom_weapon_model)
		Registers a custom weapon into the database.
		Any parameters unused should be passed as `null`.
	RegisterCustomWeaponEx(item_name, baseitem, use_original_stats, stats_function, custom_arms_model, custom_weapon_worldmodel, custom_weapon_viewmodel, custom_extra_wearable, ammotype, clipsize, ammoreserve)
		Registers a custom weapon into the database, with extra parameters.
		Any parameters unused should be passed as `null`.
	CTFPlayer.SwitchToActive(weapon)
		Finds a weapon to switch to and switches to it. If null, switches to the best valid gun, or if no gun is found, melee, or else, any.
		Note: VScript has a built-in function Weapon_Switch, which this uses.
	CTFPlayer.CreateCustomWearable(weapon, wearable_model)
		Creates a `tf_wearable` and associates it with a weapon (if valid)
		`weapon` accepts arg `null`.
	CTFPlayer.CreateArmsModel(arms_model)
		EXPERIMENTAL: Changes the class' arms model.
		Probably don't use this unless your ent count to skyrocket per player using this. Each weapon VM needs an ent to make this work!
		You need to update it every time the player touches the resupply.
		Returns arms_model as handle.
		
		
PRIVATE OPERATIONS
 Functions used only in this file or debugging. Don't use unless you know what you're doing.
	GetItemID(weapon)
		Returns netprop `m_iItemDefinitionIndex`, if valid.
		If the weapon is custom, sets bit 20 on ItemID.
		Else returns null.
	KillWepAll(weapon)
		Deletes weapon, and all wearables/associated items by that weapon.
	CTFPlayer.HasGunslinger()
		Checks if wearer is using the Gunslinger, but only for Engineer.
	CTFPlayer.AddWeapon(classname, itemindex, slot)
		Use if you want a specific weapon in a specific slot.
	CTFPlayer.UpdateArms()
		Updates all arm models for all equipped weapons. Branches from `GiveWeapon`.
	CTFPlayer.UpdateWeapon(weapon, w_model, v_model)
		Updates a weapon if it is custom or unintended for the class. Branches from `GiveWeapon`.
	CTFPlayer.AddThinkToViewModel()
		Updates the think script on tf_viewmodel. Branches from `GiveWeapon`.
	CTFPlayer.GTFW_CreateCustomWeaponModel(wearabletype, baseitem, weapon_model)
		Multiple functions. Creates tf_wearables and parents them to the player.
			wearabletype = bool, true for making tf_wearable, false for making tf_wearable_vm
			baseitem = your base item's handle. Accepts null or main_viewmodel handle for making class arms only.
			weapon_model = model.mdl
		Returns handle of the entity it creates
	GTFW_WepFix(table, player)
		Updates classnames for shotguns, pistols, saxxy, etc, before creating the weapon.
		Returns param 'table'
	ReturnWeaponBySlotBool(weapon, slot)
		Finds equip if `weapon.GetSlot()` matches `slot` parameter. Returns true if found, false if else.
	GTFW_ReturnWeaponBySlotBool(weapon, slot)
		Same as `ReturnWeaponBySlotBool`, but param `slot` takes negatives instead.
	ReturnWeaponTable(string_or_id)
		Find item by string or item ID. Don't use on its own because it doesn't have PostWepFix. Used by `CTFPlayer.ReturnWeaponTable`

Examples of Uses
	CheckItems()
		USE: In console, type "script CheckItems()" (without quotes)
			Shows slot, classname, and ID of all equipped weapons
	CTFPlayer.GiveWeapon(weapon_id_or_string)
		USE: hPlayer.GiveWeapon("Minigun")
			Gives player the stock minigun.
		USE: hPlayer.GiveWeapon(15)
			Gives player the stock Minigun.
	CTFPlayer.DeleteWeapon(weapon_handle_or_classname_or_id_or_string)
		USE: hPlayer.DeleteWeapon("MySuperCoolCustomWeapon")
			Deletes handle named MySuperCoolCustomWeapon
		USE: hPlayer.DeleteWeapon("tf_weapon_minigun")
			Deletes only miniguns
		USE: hPlayer.DeleteWeapon("Minigun")
			Deletes only stock Minigun
		USE: hPlayer.DeleteWeapon(15)
			Deletes only stock Minigun with this ID
		USE: hPlayer.DeleteWeapon(TF_WEAPONSLOTS.PRIMARY)
			Deletes all primaries
		USE: hPlayer.DeleteWeapon(TF_WEAPONSLOTS.MELEE)	
			Deletes all melees
	CTFPlayer.DisableWeapon(weapon_classname_or_id_or_string)
		USE: hPlayer.DisableWeapon(TF_WEAPONSLOTS.SECONDARY) 
			Removes all ammo from any secondary.
		USE: hPlayer.DisableWeapon("Minigun")
			Removes all ammo from the minigun
	RegisterCustomWeapon()
		--> See examples in `give_tf_weapon/custom_weapons.nut`
	RegisterCustomWeaponEx()
		--> See examples in `give_tf_weapon/custom_weapons.nut`


End instructions */

///-------------------------------------///

GTFW_VScript <- Entities.FindByName(null, "vscript_give_tf_weapon_entity");

if ( GTFW_VScript == null ) {
	::PlayerLoadoutGlobal_BonemergedArms <- {}
	::PlayerLoadoutGlobal_CustomWeaponModels_VM <- {}
	::PlayerLoadoutGlobal_CustomWeaponModels_TP <- {}
	::TF_CUSTOM_WEAPONS_REGISTRY <- {}
	::TF_CUSTOM_WEAPONS_IDs <- [0]
	
	PrecacheModel("models/weapons/c_models/c_engineer_gunslinger.mdl")
	
	GTFW_VScript <- SpawnEntityFromTable("move_rope", {targetname = "vscript_give_tf_weapon_entity", vscripts="give_tf_weapon/_master"});
	printl("Executing give_tf_weapon for the first time. Creating 'vscript_give_tf_weapon_entity' (ent: move_rope) entity.")
}

GTFW.Gamerules <- Entities.FindByClassname(null, "tf_gamerules");
if ( GTFW.Gamerules == null ) {
	GTFW_VScript <- SpawnEntityFromTable("tf_gamerules", {});
}

const GLOBAL_WEAPON_COUNT = 7
::DEVMSG <- " --> "



//-----------------------------------------------------------------------------
// Purpose: Prints error messages if CVAR_GTFW_USE_GLOBAL_ERROR_MESSAGES = true
//-----------------------------------------------------------------------------
::GTFW.DevPrint <- function(func="",message="")
{
	if ( CVAR_GTFW_USE_GLOBAL_ERROR_MESSAGES )
	{
		printl( format("give_tf_weapon %s: %s",func.tostring(),message.tostring()) )
	}
}
//-----------------------------------------------------------------------------
// Purpose: Grabs net name for player
//-----------------------------------------------------------------------------
::GTFW.GetNetName <- function(player) {
	return NetProps.GetPropString(player, "m_szNetname")
}

//-----------------------------------------------------------------------------
// Purpose: Used by events to clean up any unused entities made by this script.
//-----------------------------------------------------------------------------
::CTFPlayer.GTFW_Cleanup <- function()
{
	// Deletes any viewmodels that the script has created
	local main_viewmodel = NetProps.GetPropEntity(this, "m_hViewModel")
	AddThinkToEnt(main_viewmodel, null)	//clears script if it was being used
	if ( main_viewmodel && main_viewmodel.FirstMoveChild() )
	{
		main_viewmodel.FirstMoveChild().Kill()
	}
	if ( this.HasGunslinger() && this.GetPlayerClass() == 9 )
		main_viewmodel.SetModelSimple( GTFW_MODEL_ARMS[10] )
	else 
		main_viewmodel.SetModelSimple( GTFW_MODEL_ARMS[this.GetPlayerClass()] )
	
	local scope = null
	if( this.ValidateScriptScope() )
	{
		scope = this.GetScriptScope()
		
		if ( "Wearables" in scope ) {
			for (local i = 0; i <= scope.Wearables.len() - 1; i++) {
				if ( scope.Wearables[0].IsValid() ) {
					local w = scope.Wearables[0]
					GTFW.DevPrint("GTFW_Cleanup", "Deleting wearable... "+w+" : "+w.GetModelName() )
					w.Kill()
				}
				scope.Wearables.remove(0)
			}
		}
		
		if ( "Weapons" in scope ) {
			for (local i = 0; i <= scope.Weapons.len() - 1; i++) {
				if ( scope.Weapons[0].IsValid() ) {
					local w = scope.Weapons[0]
					GTFW.DevPrint("GTFW_Cleanup", "Deleting weapon... "+w+" : "+w.GetModelName() )
					w.Kill()
				}
				scope.Weapons.remove(0)
			}
		}
		
	}
	/*
	local loop = 0
	local wearable = null
	while ( wearable = Entities.FindByNameWithin(this, "tf_wearable_vscript*", this.GetOrigin(), 128) )
	{
		loop++
		if ( wearable && wearable.GetOwner() == this ) {
			GTFW.DevPrint("Deleting wearable... "+wearable.entindex()+" Model# "+NetProps.GetPropInt(wearable, "m_nModelIndex") )

			wearable.Kill()
		}
		if ( loop >= 15 ) { break }
	}*/
	
//clears all custom weapons
	::PlayerLoadoutGlobal_CustomWeaponModels_VM[this] <- {}
	::PlayerLoadoutGlobal_CustomWeaponModels_TP[this] <- {}
	if ( this in PlayerLoadoutGlobal_BonemergedArms ) {
		delete PlayerLoadoutGlobal_BonemergedArms[this]
	}
}

//-----------------------------------------------------------------------------
// Purpose: Weapons persisting between resupply functions
//-----------------------------------------------------------------------------
::GTFW_DeleteAllLoadouts <- function() {
	for (local i = 1; i <= Constants.Server.MAX_PLAYERS; i++)
	{
		local ply = PlayerInstanceFromIndex(i)
		if (ply == null) continue
			local Name = format("give_tf_weapon_SavedLoadout_%d",ply.entindex())
			local relay = Entities.FindByName(null, Name.tostring());
			if (relay == null) continue
				if( relay.ValidateScriptScope() )
				{
					local scope = relay.GetScriptScope()
					
					scope.Weapon_ID <- [-1]
					for (local i = 1; i < 10*7; i++)
					{
						scope.Weapon_ID.append(-1)
					}
					
					GTFW.DevPrint("DeleteAllLoadouts", "Cleaned saved inventory for "+GTFW.GetNetName(ply)+" (ent : "+Name+")")
				}
	}
}
		
		
::CTFPlayer.SaveLoadout <- function(wep=null,optional_tfclass=null) {
	
	local tfclass = null
	if ( type(optional_tfclass) != "integer" && optional_tfclass != null ) {
		GTFW.DevPrint("LoadLoadout", "Parameter 2 is not an integer.")
	}
	else {
		if ( optional_tfclass ) tfclass = optional_tfclass
		else tfclass = this.GetPlayerClass()
	}
	
	local scope = null
	local Name = format("give_tf_weapon_SavedLoadout_%d",this.entindex())
	local relay = Entities.FindByName(null, Name.tostring());
	if ( !relay ) {
		
		relay = SpawnEntityFromTable("logic_relay", {
			targetname = Name,
			spawnflags = 1,
		})
	}
	
	if( relay.ValidateScriptScope() )
	{
		scope = relay.GetScriptScope()
		
		if ( !("Weapon_ID" in scope ) ) {
			scope.Weapon_ID <- [-1]
			for (local i = 1; i < 10*7; i++)
			{
				scope.Weapon_ID.append(-1)
			}
		}
	}
	
	if ( wep == null ) {
		for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
		{
			local wep = NetProps.GetPropEntityArray(this, "m_hMyWeapons", i)
			if ( wep == null ) continue
				local table = this.ReturnWeaponTable(wep)
				local WepID = table.name
				if ( WepID == null ) WepID = table.itemID
				
				local index = tfclass*7-7+1+wep.GetSlot()
				
				scope.Weapon_ID.remove(index)
				scope.Weapon_ID.insert(index, WepID)
				
				GTFW.DevPrint("SaveLoadout", "Saved "+GTFW.GetNetName(this)+" (as TFClass "+tfclass+") (slot "+index+") : "+wep+" "+GetItemID(wep))
		}
		return relay
	}
	else if ( wep && this.ReturnWeapon(wep) ) {
		local table = this.ReturnWeaponTable(wep)
		local index = this.GetPlayerClass()*7-7+1+table.slot
		scope.Weapon_ID.remove(index)
		scope.Weapon_ID.insert(index, table.name)
		
		GTFW.DevPrint("SaveLoadout", "Saved "+GTFW.GetNetName(this)+" (as TFClass "+tfclass+") (slot "+index+") : "+this.ReturnWeapon(wep))
		return relay
	}
	if ( wep )
		GTFW.DevPrint("SaveLoadout", "Did not save "+GTFW.GetNetName(this)+" weapon "+wep)
	else
		GTFW.DevPrint("SaveLoadout", "Did not save "+GTFW.GetNetName(this)+" loadout.")
	return relay
}

::CTFPlayer.LoadLoadout <- function(loadThis=null,optional_tfclass=null) {
	
	local tfclass = null
	if ( type(optional_tfclass) != "integer" && optional_tfclass != null ) {
		GTFW.DevPrint("LoadLoadout", "Parameter 2 is not an integer.")
	}
	else {
		if ( optional_tfclass ) tfclass = optional_tfclass
		else tfclass = this.GetPlayerClass()
	}
	
	local Name = format("give_tf_weapon_SavedLoadout_%d",this.entindex())
	local relay = Entities.FindByName(null, Name.tostring());
	if ( !relay ) {
		
		relay = SpawnEntityFromTable("logic_relay", {
			targetname = Name,
			spawnflags = 1,
		})
	}
	
	if( relay.ValidateScriptScope() )
	{
		local scope = relay.GetScriptScope()
		
		if ( !("Weapon_ID" in scope ) ) {
			scope.Weapon_ID <- [-1]
			for (local i = 1; i < 10*7; i++)
			{
				scope.Weapon_ID.append(-1)
			}
		}
		if ( loadThis == null ) {
			GTFW.DevPrint("LoadLoadout", "Loaded weapons list for "+GTFW.GetNetName(this))
			for (local i = this.GetPlayerClass()*7-7+1; i < this.GetPlayerClass()*7-7+8; i++)
			{
				local wep = scope.Weapon_ID[i]
				
				GTFW.DevPrint("slot" + (i-(tfclass*7-6)).tostring(), wep)
				
				if ( wep == -1 ) continue
					this.GiveWeapon(wep)
			}
		}
		else {
			local table = this.ReturnWeaponTable(loadThis)
			if ( table == null ) {
				GTFW.DevPrint("LoadLoadout", "Failed to find weapon \x22"+loadThis+"\x22.")
				return relay
			}
			
			local i = tfclass*7-7+1+table.slot
			local wep = scope.Weapon_ID[i]
			
			if ( wep != -1 ) {
				this.GiveWeapon(wep)
				GTFW.DevPrint("LoadLoadout", "Loaded item" + table.name)
				return relay
			}
				
			GTFW.DevPrint("LoadLoadout", "Could not load slot " + (abs(i-70)).tostring())
			return relay
		}
	}
}
::CTFPlayer.DeleteLoadout <- function(wep=null, optional_tfclass=null) {
	
	local tfclass = null
	if ( type(optional_tfclass) != "integer" && optional_tfclass != null ) {
		GTFW.DevPrint("LoadLoadout", "Parameter 2 is not an integer.")
	}
	else {
		if ( optional_tfclass ) tfclass = optional_tfclass
		else tfclass = this.GetPlayerClass()
	}
	
	local Name = format("give_tf_weapon_SavedLoadout_%d",this.entindex())
	local scope = null
	local relay = Entities.FindByName(null, Name.tostring());
	if ( relay && relay.ValidateScriptScope() ) {
		
		scope = relay.GetScriptScope()
		
		if ( wep == null ) {
	
			scope.Weapon_ID <- [-1]
			for (local i = 1; i < 10*7; i++)
			{
				scope.Weapon_ID.append(-1)
			}
			
			GTFW.DevPrint("DeleteLoadout", "Deleted loadout for "+GTFW.GetNetName(this)+" (ent : "+Name+" )")
			return relay
		}
		else {
			
			local table = this.ReturnWeaponTable(wep)
			
			if ( table == null ) {
				GTFW.DevPrint("DeleteLoadout", "Failed to find weapon \x22"+wep+"\x22.")
				return relay
			}
			
			local i = tfclass*7-7+1+table.slot
			
			wep = scope.Weapon_ID[i]
			
			scope.Weapon_ID.remove(i)
			scope.Weapon_ID.insert(i, -1)
				
			GTFW.DevPrint("Deleted slot " + (abs(i-70)).tostring(), table.name)
			return relay
		}
	}
	GTFW.DevPrint("DeleteLoadout", "Could not delete "+GTFW.GetNetName(this)+" loadout.")
}

::CTFPlayer.AnnounceWeapon <- function(weapon,quality=1,prefix=null,has_string="has found:") {

// Purpose: Defining variables
	local player = this
	local playerNetName = NetProps.GetPropString(player, "m_szNetname")	//Client's Steam name
	
	local wep = player.ReturnWeaponTable(weapon)	//Convert our weapon into a table with additional information about the weapon
	if ( wep == null ) {
		GTFW.DevPrint("AnnounceWeapon", "Function failed. Could not find \x22"+weapon+"\x22. Returning null.")
		return null
	}
	
// Purpose: Defining text colors
// Here we are making the player's color change, based on their team
	local teamTextColor = null
	switch ( player.GetTeam() ) {
		case 2 : teamTextColor = "\x07FF3F3F"; break
		case 3 : teamTextColor = "\x0799CDFF"; break
		case 5 : teamTextColor = "\x05"; break
		default : teamTextColor = "\x07B2B2B2"; break
	}

// Changing the prefix for the quality item.
// If the prefix is null, make the prefix = quality
// It can be a string of your choosing, too.
	if ( prefix == null ) prefix = quality
	if ( type( prefix ) == "integer" ) {
		switch ( prefix ) {
			case 1 : {prefix = ""; break}				//	Unique
			case 2 : {prefix = "Vintage"; break}		//	Vintage
			case 3 : {prefix = "Genuine"; break}		//	Genuine
			case 4 : {prefix = "Strange"; break}		//	Strange
			case 5 : {prefix = "Unusual"; break}		//	Unusual
			case 6 : {prefix = "Haunted"; break}		//	Haunted
			case 7 : {prefix = "Collector's"; break}	//	Collector's
			case 8 : {prefix = "Decorated"; break}		//	Decorated
			case 9 : {prefix = "Self-Made"; break}		//	Community/Self-Made
			case 10 : {prefix = "Valve"; break}			//	Valve
			
			case 17 : {prefix = "Well-Designed"; break}	//	Well-Designed (unused)
			case 18 : {prefix = "Artisan"; break}			//	Artisan (unused)
			default : {prefix = ""; break}
		}
	}

//All the unique color codes for items. Phew!
	if ( type( quality ) == "integer" ) {
		switch ( quality ) {
			case 1 : {quality = "\x07FFD700"; break}	//	Unique
			case 2 : {quality = "\x07476291"; break}	//	Vintage
			case 3 : {quality = "\x074D7455"; break}	//	Genuine
			case 4 : {quality = "\x07CF6A32"; break}	//	Strange
			case 5 : {quality = "\x078650AC"; break}	//	Unusual
			case 6 : {quality = "\x0738F3AB"; break}	//	Haunted
			case 7 : {quality = "\x07AA0000"; break}	//	Collector's
			case 8 : {quality = "\x07FAFAFA"; break}	//	Decorated
			case 9 : {quality = "\x0770B04A"; break}	//	Community/Self-Made
			case 10 : {quality = "\x07A50F79"; break}	//	Valve
			
			case 11 : {quality = "\x07B0C3D9"; break}	//	Civilian Grade
			case 12 : {quality = "\x075E98D9"; break}	//	Freelance Grade
			case 13 : {quality = "\x074B69FF"; break}	//	Mercenary Grade
			case 14 : {quality = "\x078847FF"; break}	//	Commando Grade
			case 15 : {quality = "\x07D32CE6"; break}	//	Assassin Grade 
			case 16 : {quality = "\x07EB4B4B"; break}	//	Elite Grade
			
			case 17 : {quality = "\x078D834B"; break}	//	Well-Designed (unused)
			case 18 : {quality = "\x0770550F"; break}	//	Artisan (unused)
			default : {quality = "\x07B2B2B2"; break}
		}
	}
//Finishing touches
	//If prefix has anything in the string, add a space
	local space = ""
	if ( prefix.len() ) space = " "
	
	//format item name
	local display = format("%s%s%s%s", quality, prefix, space, wep.name)
	
//Voila! Text is displayed for all players!
	ClientPrint(null,3,format("%s%s \x01%s %s",teamTextColor,playerNetName,has_string,display) )
}
//-----------------------------------------------------------------------------
// Purpose: Deletes weapons and all wearables tied to it.
//-----------------------------------------------------------------------------
::KillWepAll <- function(wep=null)
{
	if ( wep == null ) {
		GTFW.DevPrint("KillWepAll", "Function failed. Parameter is null.")
		return null
	}
	local wearable = NetProps.GetPropEntity(wep, "m_hExtraWearable")
	if ( wearable ) {
		wearable.Kill()
	}
	wearable = NetProps.GetPropEntity(wep, "m_hExtraWearableViewModel")
	if ( wearable ) {
		wearable.Kill()
	}
	wep.Kill()
}


//-----------------------------------------------------------------------------
// Purpose: For testing if weapon is custom or is unintended
// If "bool" set to "true", adds the flag to make the weapon either custom or unintended weapon
//-----------------------------------------------------------------------------
::IS_CUSTOM <- function(weapon=null, bool=false)
{
	local IS_CUSTOM = abs(NetProps.GetPropInt(weapon, "m_AttributeManager.m_Item.m_iItemIDHigh"))
	
	if ( bool ) {
		NetProps.SetPropInt(weapon, "m_AttributeManager.m_Item.m_iItemIDHigh", abs(IS_CUSTOM) | (1 << 6) )	//means our weapon is custom
		NetProps.SetPropInt(weapon, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", (GetItemID(weapon) | (1 << 20)) )
	}
	if ( IS_CUSTOM & (1 << 6) ) {
		return true
	}
	else { return false }
}
::IS_UNINTENDED <- function(weapon=null, bool=false)
{
	local UNINTENDED_ID = abs(NetProps.GetPropInt(weapon, "m_AttributeManager.m_Item.m_iItemIDHigh"))
	
	if ( bool ) {
		NetProps.SetPropInt(weapon, "m_AttributeManager.m_Item.m_iItemIDHigh", abs(UNINTENDED_ID) | (1 << 5) )	//means our weapon is unintended
	}
	if ( UNINTENDED_ID & (1 << 5) ) {
		return true
	}
	else { return false }
}


//-----------------------------------------------------------------------------
// Purpose: Finds a weapon to switch to and switches to it.
//	-> See top of script for more info
//-----------------------------------------------------------------------------
::CTFPlayer.SwitchToActive <- function(NewGun=null)
{
// Finds a weapon to switch to if:
// The weapon is not on the list of Misc weapons, is not a melee, and weight is greater or equal to 2,
// if not, then checks if the weapon is not in slot5, nor slot6, and is a melee,
// if not, just switches to any weapon available.
	local SwitchTo = NewGun
	local wep = null
	local SearchTimes = 0
	if ( !SwitchTo ) {
		for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
		{
			wep = NetProps.GetPropEntityArray(this, "m_hMyWeapons", i)
			if ( SearchTimes == 0 && wep && SearchMiscWeapons.find(wep.GetClassname()) == null && !wep.IsMeleeWeapon() && wep.GetWeight() >= 2 && wep.HasPrimaryAmmo() && wep.HasSecondaryAmmo() ) {
				SwitchTo = wep
				break
			}
			else if ( SearchTimes == 1 && wep && wep.IsMeleeWeapon() && wep.GetWeight() >= 2 ) {
				SwitchTo = wep
				break
			}
			else if ( SearchTimes == 2 && wep && wep.IsMeleeWeapon() ) {
				SwitchTo = wep
				break
			}
			else if ( SearchTimes == 3 && wep ) {
				SwitchTo = wep
				break
			}
			else if ( i == GLOBAL_WEAPON_COUNT - 1 && SearchTimes < 3 ) {
				i = 0
				SearchTimes++
			}
		}
	}
	
	this.Weapon_Switch(SwitchTo)
	return SwitchTo
}


//-----------------------------------------------------------------------------
// Purpose: Finds Gunslinger on player, but only for Engineer. If found, returns true.
//-----------------------------------------------------------------------------
::CTFPlayer.HasGunslinger <- function()
{
	if ( this.GetPlayerClass() == 9 )
	{
		for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
		{
			local wep = NetProps.GetPropEntityArray(this, "m_hMyWeapons", i)
			
			if ( wep && wep.GetClassname() == "tf_weapon_robot_arm" )
			{
				return true
			}
		}
	}
	return false
}


//-----------------------------------------------------------------------------
// Purpose: Find slot of weapon by classname
//-----------------------------------------------------------------------------

::ReturnWeaponSlot <- function(classname) {
	if ( SearchPrimaryWeapons.find(classname) ) {
		return 0
	}
	else if ( SearchSecondaryWeapons.find(classname) ) {
		return 1
	}
	else if ( SearchMeleeWeapons.find(classname) ) {
		return 2
	}
	else if ( SearchSlot3Weapons.find(classname) ) {
		return 3
	}
	else if ( SearchSlot4Weapons.find(classname) ) {
		return 4
	}
	else if ( SearchSlot5Weapons.find(classname) ) {
		return 5
	}
	else if ( SearchSlot6Weapons.find(classname) ) {
		return 6
	}
	else {
		return -1
	}
}
	
//-----------------------------------------------------------------------------
// Purpose: Searches all items under tables.nut. 
// Same as handle.ReturnWeaponTable() but without the handle and post fixes to classnames/ammo.
// Used by handle.ReturnWeaponTable().
//-----------------------------------------------------------------------------
::GTFW_SearchEquipReg <- function(baseitem=null)
{
	if ( baseitem == null ) {
		return
	}
	local Continue = true
	local ID	= null
	local wepTable = null
	
	foreach (exists in TF_CUSTOM_WEAPONS_REGISTRY)
	{
		if ( exists.itemString == baseitem )
		{
			ID = exists.itemID
			wepTable = GTFW_PreWepFix(exists, ID)
			
			Continue = false
			return wepTable
		}
		else if ( exists.itemID == baseitem )
		{
			ID = exists.itemID
			wepTable = GTFW_PreWepFix(exists, ID)
				
			Continue = false
			return wepTable
		}
	}
	if ( Continue )
	{
		foreach (exists in TF_WEAPONS_ALL)
		{
			if ( exists.itemString == baseitem || exists.itemString2 == baseitem )
			{
				ID = exists.itemID
				wepTable = GTFW_PreWepFix(exists, ID)
				
				Continue = false
				return wepTable
			}
			else if ( exists.itemID == baseitem )
			{
				ID = exists.itemID
				
				wepTable = GTFW_PreWepFix(exists, ID)
				
				Continue = false
				return wepTable
			}
		}
		if ( Continue )
		{
			foreach (exists in TF_WEAPONS_ALL_WARPAINTSnBOTKILLERS)
			{
				if ( exists.itemID == baseitem || exists.itemID2 == baseitem || exists.itemID3 == baseitem
				|| exists.itemID4 == baseitem || exists.itemID5 == baseitem || exists.itemID6 == baseitem
				|| exists.itemID7 == baseitem || exists.itemID8 == baseitem || exists.itemID9 == baseitem
				|| exists.itemID10 == baseitem || exists.itemID11 == baseitem || exists.itemID12 == baseitem
				|| exists.itemID13 == baseitem)
				{
					if ( exists.itemID == baseitem ) {
						ID = exists.itemID
					}
					else if ( exists.itemID2 == baseitem )
						ID = exists.itemID2
					else if ( exists.itemID3 == baseitem )
						ID = exists.itemID3
					else if ( exists.itemID4 == baseitem )
						ID = exists.itemID4
					else if ( exists.itemID5 == baseitem )
						ID = exists.itemID5
					else if ( exists.itemID6 == baseitem )
						ID = exists.itemID6
					else if ( exists.itemID7 == baseitem )
						ID = exists.itemID7
					else if ( exists.itemID8 == baseitem )
						ID = exists.itemID8
					else if ( exists.itemID9 == baseitem )
						ID = exists.itemID9
					else if ( exists.itemID10 == baseitem )
						ID = exists.itemID10
					else if ( exists.itemID11 == baseitem )
						ID = exists.itemID11
					else if ( exists.itemID12 == baseitem )
						ID = exists.itemID12
					else if ( exists.itemID13 == baseitem )
						ID = exists.itemID13
					
					wepTable = GTFW_PreWepFix(exists, ID)
					
					return wepTable
				}
			}
		}
	}
}

//-----------------------------------------------------------------------------
// Purpose: Finds a weapon based on handle, slot, ID, classnames, or string.
// Requires player handle to start the function.
//-----------------------------------------------------------------------------
::CTFPlayer.ReturnWeaponTable <- function(weapon)
{
	if ( weapon == null ) {
		GTFW.DevPrint("ReturnWeaponTable", "Function failed. Parameter is Null. Returning null.")
		return null
	}
	local ItemID = null
	if ( type(weapon) == "instance" ) {
		ItemID = GetItemID(weapon)
	}
	local baseitem = weapon
	
	if ( NetProps.HasProp(weapon, "m_AttributeManager.m_Item.m_iItemDefinitionIndex") ) { //if a weapon handle
		if ( IS_CUSTOM(weapon) ) { //if custom
			ItemID = ItemID | (1 << 20)  //sets bit 20 for searching in ReturnWeaponTable
		}
	}
	
	if ( SearchBySlotsParameters.find(weapon) && weapon != 0 ) {
		for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
		{
			local wep = NetProps.GetPropEntityArray(this, "m_hMyWeapons", i)
			if ( wep )
			{
				if ( GTFW_ReturnWeaponBySlotBool(wep, weapon) )
				{
					ItemID = GetItemID(wep)
					baseitem = GTFW_SearchEquipReg(ItemID)
					baseitem = this.GTFW_PostWepFix(baseitem)
					return baseitem
				}
			}
		}
	}
	else if ( SearchAllWeapons.find(weapon) ) {
		for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
		{
			local wep = NetProps.GetPropEntityArray(this, "m_hMyWeapons", i)

			if ( wep )
			{
				if ( wep.GetClassname() == weapon )
				{
					ItemID = GetItemID(wep)
					baseitem = GTFW_SearchEquipReg(ItemID)
					baseitem = this.GTFW_PostWepFix(baseitem)
					return baseitem
				}
			}
		}
	}
	else {
		if ( baseitem == null ) {
			GTFW.DevPrint("ReturnWeaponTable", "Function failed. Param \x22baseitem\x22 is Null. Returning null.")
			return null
		}
		baseitem = GTFW_SearchEquipReg(weapon)
		
		if ( baseitem ) {
			baseitem = this.GTFW_PostWepFix(baseitem)
			return baseitem
		}
		else {
			baseitem = GTFW_SearchEquipReg(ItemID)
			if ( baseitem ) {
				baseitem = this.GTFW_PostWepFix(baseitem)
				return baseitem
			}
		}
	}
	
	GTFW.DevPrint("ReturnWeaponTable", "Function failed. Could not find weapon. Returning null.")
	return null
}

//-----------------------------------------------------------------------------
// Purpose: Pre-Fixes up all items when found by handle.ReturnWeaponTable().
// It shoves everything that's needed in a weapon into a table to be accessed by you, the user!
//-----------------------------------------------------------------------------
::GTFW_PreWepFix <- function(exists, ID)
{
	local tf_class	= null
	local func = null
	local clear_stats = null
	local w_model = null
	local v_model = null
	local class_arms = null
	if ( "tf_class" in exists ) {
		tf_class = exists.tf_class
	}
	else { 
		tf_class = 0
	}
	
	if ( "func" in exists ) {
		func = exists.func
		clear_stats = exists.clear_stats
		w_model = exists.w_model
		v_model = exists.v_model
		class_arms = exists.class_arms
	}
	local itemName	= null
	if ( "itemString" in exists ) {
		itemName	= exists.itemString
	}
	local slot		= exists.slot
	local className	= exists.className
	local itemID	= ID //& ~(1 << 20)	//removes custom weapon bit
	local ammo_type	= exists.ammo_type
	local clip		= exists.clip
	local reserve	= exists.reserve
	local wearable	= exists.wearable
	
	local baseitem	= TF_WEAPONS_BASE(tf_class, slot, className, itemID, itemName, ammo_type, clip, reserve, w_model, v_model, wearable, clear_stats, func, class_arms)
	
	return baseitem
}


//-----------------------------------------------------------------------------
// Purpose: Post-Fixes up all items found by handle.ReturnWeaponTable.
// Things like what the classname is, what type of shotgun it is, etc.
//-----------------------------------------------------------------------------
::CTFPlayer.GTFW_PostWepFix <- function(baseitem)
{
	if ( baseitem == null) {
		return
	}
	local baseC = baseitem.className
	
	if ( baseC == "demoshield" )
	{
		baseitem.className = "tf_wearable_demoshield"
	}
	else if ( baseC == "razorback" )
	{
		baseitem.className = "tf_wearable_razorback"
	}
	else if ( baseC == "tf_wearable" )
	{
		baseitem.className = baseC
	}
	else if ( baseC == "saxxy" ) {
		baseitem.className = GTFW_Saxxy[this.GetPlayerClass()]
	}
	else if ( baseC == "pistol" ) {
		if ( this.GetPlayerClass() == Constants.ETFClass.TF_CLASS_SCOUT ) {
			baseitem.className = "tf_weapon_pistol_scout"
			baseitem.reserve		= 36
			baseitem.tf_class 	=	Constants.ETFClass.TF_CLASS_SCOUT
		}
		else if ( this.GetPlayerClass() == Constants.ETFClass.TF_CLASS_ENGINEER ) {
			baseitem.className = "tf_weapon_pistol"
			baseitem.reserve		= 36
			baseitem.tf_class 	=	Constants.ETFClass.TF_CLASS_ENGINEER
		}
		else {
			baseitem.className = "tf_weapon_pistol"
			baseitem.reserve		= 36
			baseitem.tf_class 	=	Constants.ETFClass.TF_CLASS_ENGINEER
		}
	}
	else if ( baseC == "shotgun" ) {
		if ( this.GetPlayerClass() == Constants.ETFClass.TF_CLASS_ENGINEER ) {
			baseitem.className	= "tf_weapon_shotgun_primary"
			baseitem.slot = 0
			baseitem.tf_class 	=	Constants.ETFClass.TF_CLASS_ENGINEER
			baseitem.ammo_type	=	TF_AMMO.PRIMARY
		}
		else if ( this.GetPlayerClass() == Constants.ETFClass.TF_CLASS_HEAVYWEAPONS ) {
			baseitem.className	= "tf_weapon_shotgun_hwg"
			baseitem.tf_class 	=	Constants.ETFClass.TF_CLASS_HEAVYWEAPONS
		}
		else if ( this.GetPlayerClass() == Constants.ETFClass.TF_CLASS_SOLDIER ) {
			baseitem.className	= "tf_weapon_shotgun_soldier"
			baseitem.tf_class 	=	Constants.ETFClass.TF_CLASS_SOLDIER
		}
		else if ( this.GetPlayerClass() == Constants.ETFClass.TF_CLASS_PYRO ) {
			baseitem.className	= "tf_weapon_shotgun_pyro"
			baseitem.tf_class 	=	Constants.ETFClass.TF_CLASS_PYRO
		}
		else if ( this.GetPlayerClass() == Constants.ETFClass.TF_CLASS_DEMOMAN ) {
			baseitem.className	= "tf_weapon_shotgun_soldier"
			baseitem.tf_class 	=	Constants.ETFClass.TF_CLASS_SOLDIER
		}
		else if ( this.GetPlayerClass() == Constants.ETFClass.TF_CLASS_MEDIC ) {
			baseitem.className	= "tf_weapon_shotgun_primary"
			baseitem.slot = 0
			baseitem.tf_class 	=	Constants.ETFClass.TF_CLASS_ENGINEER
			baseitem.ammo_type	=	TF_AMMO.PRIMARY
		}
		else if ( this.GetPlayerClass() == Constants.ETFClass.TF_CLASS_SPY ) {
			baseitem.className	= "tf_weapon_shotgun_primary"
			baseitem.slot = 0
			baseitem.tf_class 	=	Constants.ETFClass.TF_CLASS_ENGINEER
			baseitem.ammo_type	=	TF_AMMO.PRIMARY
		}
		else if ( this.GetPlayerClass() == Constants.ETFClass.TF_CLASS_SCOUT ) {
			baseitem.className	= "tf_weapon_shotgun_primary"
			baseitem.slot = 0
			baseitem.tf_class 	=	Constants.ETFClass.TF_CLASS_ENGINEER
			baseitem.ammo_type	=	TF_AMMO.PRIMARY
		}
		else {
			baseitem.className	= "tf_weapon_shotgun_pyro"
			baseitem.tf_class 	=	Constants.ETFClass.TF_CLASS_PYRO
		}
	}
	else if ( baseC == "katana" ) {
		if ( this.GetPlayerClass() == Constants.ETFClass.TF_CLASS_DEMOMAN ) {
			baseitem.className = "tf_weapon_katana"
			baseitem.tf_class 	=	Constants.ETFClass.TF_CLASS_DEMOMAN
		}
	}
	else if ( baseC == "tf_weapon_revolver" ) {
		if ( this.GetPlayerClass != Constants.ETFClass.TF_CLASS_SPY )
		{
			baseitem.ammo_type	=	TF_AMMO.SECONDARY
		}
	}
	else { baseitem.className = "tf_weapon_"+baseitem.className	}
	
	return baseitem
}


//-----------------------------------------------------------------------------
//Base for making weapons. Netprops compiled by ficool2.
//-----------------------------------------------------------------------------
::CTFPlayer.AddWeapon <- function(classname, itemindex, slot)
{
	if ( itemindex & (1 << 20) ) {
		itemindex = itemindex & ~(1 << 20)
	}
	local weapon = Entities.CreateByClassname(classname)
	
	NetProps.SetPropInt(weapon, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", itemindex)
//	NetProps.SetPropInt(weapon, "m_AttributeManager.m_Item.m_iEntityQuality", RandomInt(1,100) )	//Can't set due to security risks set by vscript
	NetProps.SetPropInt(weapon, "m_AttributeManager.m_Item.m_iEntityLevel", 0)
	NetProps.SetPropInt(weapon, "m_AttributeManager.m_Item.m_bInitialized", 1)
	//NetProps.SetPropInt(weapon, "m_AttributeManager.m_Item.m_iAccountID", -1)	//Can't find due to security risks set by vscript
	
	NetProps.SetPropInt(weapon, "m_bClientSideAnimation", 1)
	NetProps.SetPropInt(weapon, "m_fEffects", 129)
	//NetProps.SetPropInt(weapon, "m_iState", 2)	//not needed
	NetProps.SetPropInt(weapon, "m_CollisionGroup", 11)
	
	local curTime = Time()
	NetProps.SetPropFloat(weapon, "LocalActiveWeaponData.m_flNextPrimaryAttack", curTime)
	NetProps.SetPropFloat(weapon, "LocalActiveWeaponData.m_flNextSecondaryAttack", curTime)
	NetProps.SetPropFloat(weapon, "LocalActiveWeaponData.m_flTimeWeaponIdle", curTime)
	
	NetProps.SetPropInt(weapon, "m_bValidatedAttachedEntity", 1)
	//NetProps.SetPropInt(weapon, "m_AttributeManager.m_Item.m_bOnlyIterateItemViewAttributes", 1)	// Removes all pre-existing stats of the weapon.
	NetProps.SetPropInt(weapon, "m_AttributeManager.m_iReapplyProvisionParity", 1)
	
	weapon.SetAbsOrigin(Vector(0,0,0))
	weapon.SetAbsAngles(QAngle(0,0,0))
	
	NetProps.SetPropInt(weapon, "m_iTeamNum", this.GetTeam())
	NetProps.SetPropEntity(weapon, "m_hOwnerEntity", this)
	NetProps.SetPropEntity(weapon, "m_hOwner", this)
	weapon.SetOwner(null);
	weapon.SetOwner(this);
				
	Entities.DispatchSpawn(weapon)
	weapon.ReapplyProvision()
	
	local solidFlags = NetProps.GetPropInt(weapon, "m_Collision.m_usSolidFlags");
	NetProps.SetPropInt(weapon, "m_Collision.m_usSolidFlags", solidFlags | Constants.FSolid.FSOLID_NOT_SOLID);
	
	solidFlags = NetProps.GetPropInt(weapon, "m_Collision.m_usSolidFlags");
	NetProps.SetPropInt(weapon, "m_Collision.m_usSolidFlags", solidFlags & ~(Constants.FSolid.FSOLID_TRIGGER));
	
	if ( classname != "tf_wearable_demoshield" || classname != "tf_wearable" || classname != "tf_wearable_razorback" )
	{
		NetProps.SetPropEntityArray(this, "m_hMyWeapons", weapon, slot)
	}
	weapon.SetLocalOrigin(this.GetLocalOrigin())
	weapon.SetLocalAngles(this.GetAbsAngles())
	DoEntFire("!self", "SetParent", "!activator", 0, this, weapon)
	NetProps.SetPropInt(weapon, "m_MoveType", 0)
	

//If added weapon is not intended to be used by player's tfclass, mark it as unintended for the tfclass, to fix VM arms
	local addedWep = this.ReturnWeaponTable(itemindex)
	if ( addedWep == null ) {
		GTFW.DevPrint("AddWeapon", "Function failed at finding item index \x22"+itemindex+"\x22. Returning null.")
		return null
	}
	if ( this.GetPlayerClass() != addedWep.tf_class && addedWep.tf_class != 0 )
	{
		IS_UNINTENDED(weapon, true)
	}

	if ( classname == "tf_weapon_builder")	//Engineer toolbox
	{
		NetProps.SetPropInt(weapon, "BuilderLocalData.m_iObjectType", 0)
		NetProps.SetPropInt(weapon, "m_iSubType", 0)
		NetProps.SetPropInt(weapon, "BuilderLocalData.m_aBuildableObjectTypes.000", 1)
		NetProps.SetPropInt(weapon, "BuilderLocalData.m_aBuildableObjectTypes.001", 1)
		NetProps.SetPropInt(weapon, "BuilderLocalData.m_aBuildableObjectTypes.002", 1)
		NetProps.SetPropInt(weapon, "BuilderLocalData.m_aBuildableObjectTypes.003", 0)
	}

	GTFW.DevPrint("AddWeapon", "Initialized \x22"+weapon+"\x22.")
	return weapon
}

//-----------------------------------------------------------------------------
// Purpose: Fixes ammo.
// There is an invisible tf_wearable that tracks ammo for custom weapons, and DisableWeapon/EnableWeapon
// Same goes for metal ammotype weapons like Widowmaker, Wrenches, PDAs etc
//-----------------------------------------------------------------------------
::CTFPlayer.UpdateAmmoFix <- function(weapon,WepTFClass,AmmoType,ClipSize,AmmoReserve)
{
	
	local WepC = weapon.GetClassname()
	
	if ( AmmoType > TF_AMMO.NONE && AmmoReserve >= 0 ) {
		
	//initializes tf_wearable for AmmoFix
		local Name = format("tf_wearable_vscript_%d_%d_%s",this.entindex(),65535,"AMMOFIX")
		local AmmoFix = Entities.FindByName(null, Name.tostring());
		if ( !AmmoFix ) {
			AmmoFix = this.CreateCustomWearable("models/empty.mdl")
		}
		
		local aThreshold = 0
		local aPerClass = 0
	// Purpose: Updates primary ammo reserve
		if ( AmmoType == TF_AMMO.PRIMARY ) {
			aPerClass = AmmoReserve == -1 ? TF_AMMO_PER_CLASS_PRIMARY[WepTFClass] : AmmoReserve
			aThreshold = TF_AMMO_PER_CLASS_PRIMARY[this.GetPlayerClass()]
			AmmoFix.AddAttribute("hidden primary max ammo bonus", aPerClass.tofloat() / aThreshold.tofloat(), -1)
		}
	// Purpose: Updates secondary ammo reserve
		else if ( AmmoType == TF_AMMO.SECONDARY ) {
			aPerClass = AmmoReserve == -1 ? TF_AMMO_PER_CLASS_SECONDARY[WepTFClass] : AmmoReserve
			aThreshold = TF_AMMO_PER_CLASS_SECONDARY[this.GetPlayerClass()]
			weapon.AddAttribute("hidden secondary max ammo penalty", aPerClass.tofloat() / aThreshold.tofloat(), -1)
		}
	// Purpose: Updates GRENADES1 ammo reserve
		else if ( AmmoType == TF_AMMO.GRENADES1 ) {
			aThreshold = 1
			AmmoFix.AddAttribute("maxammo grenades1 increased", (AmmoReserve.tofloat() / aThreshold.tofloat()), -1)
		}
	// Purpose: Updates metal capacity
		else if ( AmmoType == TF_AMMO.METAL ) {
			aThreshold = 200
			AmmoFix.AddAttribute("maxammo metal increased", (AmmoReserve.tofloat() / aThreshold.tofloat()), -1)
		}
			
		local FinalAmmoReserve = aThreshold * (AmmoReserve.tofloat() / aThreshold.tofloat())
		
	// Purpose: Applies all attributes to player
		AmmoFix.ReapplyProvision()
		
	// Purpose: Sets ammo reserve
		if ( AmmoType == TF_AMMO.METAL && EntityOutputs.HasAction(AmmoFix, "OnUser3") == false && this.GetPlayerClass() != Constants.ETFClass.TF_CLASS_ENGINEER ) {
			EntityOutputs.AddOutput(AmmoFix, "OnUser3", "", "", "", 0.0, -1)	//used to flag for Metal ammo so we don't update twice
			NetProps.SetPropIntArray(this, "m_iAmmo", FinalAmmoReserve, TF_AMMO.METAL)
		}
		else {
			NetProps.SetPropIntArray(this, "m_iAmmo", FinalAmmoReserve, AmmoType)
		}
	}
	// Purpose: Sets ammo clip
	// It seems like having both helps account for weapons that don't use the atomic upgrade, but atomic seems to supersede the %clipsize penalty for what ever reason
	if ( ClipSize >= 0 ) {
		if ( (ClipSize.tofloat() / weapon.GetMaxClip1().tofloat()) != 1 ) {
			weapon.AddAttribute("clip size upgrade atomic", 0 - (weapon.GetMaxClip1() - ClipSize), -1)
			weapon.AddAttribute("clip size penalty HIDDEN", ClipSize.tofloat() / weapon.GetMaxClip1().tofloat(), -1)
		}
		weapon.SetClip1(ClipSize)
	}
	
	return weapon
}

//-----------------------------------------------------------------------------
// Purpose: Used to update viewmodel arms.
//	-> See top of script for more info
//-----------------------------------------------------------------------------
::CTFPlayer.UpdateArms <- function(givenwep=null, force_class_arms=null)
{
	if ( !CVAR_GTFW_USE_VIEWMODEL_FIX ) {
		GTFW.DevPrint("UpdateArms", "CVAR_GTFW_USE_VIEWMODEL_FIX = false. UpdateArms cancelled.")
		return
	}
	
/* Purpose: This is the special VM fix.
 it creates class arms that bonemerge with VM
*/
		
// Table for bonemerged class arms, per player.
// Adds a class arms model if nothing has parented to the main_viewmodel
	if ( !( this in PlayerLoadoutGlobal_BonemergedArms ) ) {
		local main_viewmodel = NetProps.GetPropEntity(this, "m_hViewModel")
		local wearable_handle = this.GTFW_CreateCustomWeaponModel(false, main_viewmodel, GTFW_MODEL_ARMS[this.GetPlayerClass()] )	// creates player class' arms, parents and does all that stuff
	/*	Purpose: Initialize tables if they aren't already made
		So we don't create more arms than necessary
		Puts into a table "player = arms model" for easy tracking
	*/
		::PlayerLoadoutGlobal_BonemergedArms[this] <- wearable_handle
	}
	
// Purpose: Writes to the tables above, which keep the class arms the weapon needs to update to when switched to.
// Ficool2's code. Thank you!
	for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
	{
		local wep = NetProps.GetPropEntityArray(this, "m_hMyWeapons", i)
		
		if ( wep )
		{
			local baseitem = this.ReturnWeaponTable(GetItemID(wep)&~(1<<20))
			
			if ( baseitem ) {
				//GTFW.DevPrint("UpdateArms", "+wep+" "+baseitem.itemID+" "+givenwep+" "+GetItemID(givenwep)+" "+(GetItemID(givenwep)&~(1<<20)))
				local model = null
				if ( force_class_arms && baseitem.itemID == (GetItemID(givenwep)&~(1<<20) ))
					model = force_class_arms
				else if ( baseitem.className == "tf_weapon_pda_spy" )
					model = "models/weapons/v_models/v_pda_spy.mdl"
				else if ( baseitem.className == "tf_weapon_invis" && baseitem.itemID == 30 )
					model = "models/weapons/v_models/v_watch_spy.mdl"
				else if ( baseitem.className == "tf_weapon_invis" && baseitem.itemID == 59 )
					model = "models/weapons/v_models/v_watch_pocket_spy.mdl"
				else if ( baseitem.className == "tf_weapon_invis" && baseitem.itemID == 60 )
					model = "models/weapons/v_models/v_watch_leather_spy.mdl"
				else if ( baseitem.className == "tf_weapon_invis" && baseitem.itemID == 297 )
					model = "models/weapons/v_models/v_ttg_watch_spy.mdl"
				else if ( baseitem.className == "tf_weapon_invis" && baseitem.itemID == 947 )
					model = "models/workshop_partner/weapons/v_models/v_hm_watch/v_hm_watch.mdl"
				else if ( this.HasGunslinger() )
					model = GTFW_MODEL_ARMS[10]
				else if ( baseitem.class_arms )
					model = baseitem.class_arms
				else if ( baseitem.tf_class == 0 || this.GetPlayerClass() == baseitem.tf_class )
					model = GTFW_MODEL_ARMS[this.GetPlayerClass()]
				else
					model = GTFW_MODEL_ARMS[baseitem.tf_class]
				
				local modelIndex = GetModelIndex( model )
				wep.SetModelSimple( model )
				wep.SetCustomViewModelModelIndex( modelIndex )
				NetProps.SetPropInt( wep, "m_iViewModelIndex", modelIndex )
			}
		}
	}
}


//-----------------------------------------------------------------------------
//	Purpose: Updates all weapon models.
//	-> See top of script for more info
//-----------------------------------------------------------------------------
 
::CTFPlayer.UpdateWeapon <- function(weapon, w_model, v_model)
{
// Only update if the weapon is custom or is unintended for tfclass
	if ( IS_CUSTOM(weapon) || IS_UNINTENDED(weapon) )
	{
		local wearable_handle = this.GTFW_CreateCustomWeaponModel(false, weapon, v_model)	// creates weapon in viewmodel, parents and does all that stuff

	//Mr. Burguers helped me with this part. Thanks Mr B!
		local wep = weapon.GetClassname()
		if ( !( this in PlayerLoadoutGlobal_CustomWeaponModels_VM ) ) {
			PlayerLoadoutGlobal_CustomWeaponModels_VM[this] <- {}
		}
		if ( wep in PlayerLoadoutGlobal_CustomWeaponModels_VM[this] )
		{
			local wepModel = PlayerLoadoutGlobal_CustomWeaponModels_VM[this][wep]
			if ( wepModel.IsValid() ) {
				KillWepAll(wepModel)
			}
			delete PlayerLoadoutGlobal_CustomWeaponModels_VM[this][wep]
		}
		PlayerLoadoutGlobal_CustomWeaponModels_VM[this][wep] <- wearable_handle
		
		if ( IS_CUSTOM(weapon) )	//only make third person model if weapon is custom
		{
			wearable_handle = this.GTFW_CreateCustomWeaponModel(true, weapon, w_model)	// creates thirdperson/world model, parents and does all that stuff
			
			if ( !( this in PlayerLoadoutGlobal_CustomWeaponModels_TP ) ) {
				PlayerLoadoutGlobal_CustomWeaponModels_TP[this] <- {}
			}
			if ( wep in PlayerLoadoutGlobal_CustomWeaponModels_TP[this] )
			{
				local wepModel = PlayerLoadoutGlobal_CustomWeaponModels_TP[this][wep]
				if ( wepModel.IsValid() ) {
					KillWepAll(wepModel)
				}
				delete PlayerLoadoutGlobal_CustomWeaponModels_TP[this][wep]
			}
			
			PlayerLoadoutGlobal_CustomWeaponModels_TP[this][wep] <- wearable_handle
		}
	}
	return weapon
}


//-----------------------------------------------------------------------------
// Purpose: Updates viewmodel think script.
// Clears and updates the viewmodel think script for weapon switching, enabling/disabling visibility of custom weapons.
//-----------------------------------------------------------------------------
::CTFPlayer.AddThinkToViewModel <- function()
{
	if ( !CVAR_GTFW_USE_VIEWMODEL_FIX ) {
		return
	}
	local main_viewmodel = NetProps.GetPropEntity(this, "m_hViewModel")
	local wearable_handle = null
	local wep = null
	local player = this
	local wearable = null
		
	local DisableDrawQueue = null
	local asdf = null
	local WEP_TAUNT_FIX = false
	local last_active_building = null
	
	//Checks game state to see if in humiliation mode
	::GTFW_GameState <- function(player) {
		local RoundState = NetProps.GetPropInt(GTFW.Gamerules, "teamplayroundbased_gamerules_data.m_iRoundState")
		local WinningTeam = NetProps.GetPropInt(GTFW.Gamerules, "teamplayroundbased_gamerules_data.m_iWinningTeam")
		if ( RoundState == 5 && WinningTeam != player.GetTeam() )
			return true
		else 
			return false
	}
	
	local playerscope = null
	if( player.ValidateScriptScope() )
	{
		playerscope = player.GetScriptScope()
	}
// Think script itself.
// Reads from several tables to find weapon's class arms.
	if( main_viewmodel.ValidateScriptScope() )
	{
		const THINK_VMFIX_DELAY = 0.01
		
		local entscriptname = "THINK_VM_FIX_"+this.tostring()
		local entityscript = main_viewmodel.GetScriptScope()
		entityscript[entscriptname] <- function()
		{
		// Fixes taunts
		// It clears the parent of the original weapon, and makes it invisible.
		// It is reparented inside the DisableDrawQueue section (a few lines down)
			if ( ( player.InCond(7) && WEP_TAUNT_FIX ) || ( GTFW_GameState(player) && WEP_TAUNT_FIX ) ) {
				WEP_TAUNT_FIX = false
				wep = player.GetActiveWeapon()
				
				if ( IS_CUSTOM(wep) || IS_UNINTENDED(wep) )
				{
					NetProps.SetPropEntity(DisableDrawQueue, "m_hWeaponAssociatedWith", wep)
					NetProps.SetPropInt(wep, "m_fEffects", 161)
					DoEntFire("!self", "Clearparent", "", 0, null, wep)
					DoEntFire("!self", "DisableShadow", "", 0, null, wep)
					wep.EnableDraw()
					NetProps.SetPropInt(wep, "m_iState", 0)
					wep = null
				}
				
				for (local i = 0; i <= playerscope.Wearables.len() - 1; i++) {
					if ( playerscope.Wearables[i].IsValid() ) {
						NetProps.SetPropInt(playerscope.Wearables[i], "m_AttributeManager.m_Item.m_bInitialized", 0)
						EntFireByHandle(playerscope.Wearables[i], "RunScriptCode", "NetProps.SetPropInt(self, \x22m_AttributeManager.m_Item.m_bInitialized\x22, 1)", 0.0, playerscope.Wearables[i], playerscope.Wearables[i])
					}
				}
			}
		// updates weapons' visibility based on which one is being used
			else if ( !GTFW_GameState(player) && player.GetActiveWeapon() && !player.InCond(7) && player.GetActiveWeapon() != wep || ( NetProps.HasProp(wep, "m_hObjectBeingBuilt") && NetProps.GetPropEntity(wep, "m_hObjectBeingBuilt") != last_active_building ) )
			{			
				wep = player.GetActiveWeapon()
				local wepC = wep.GetClassname()
				
		// This fixes Engineer's Toolbox
				if ( NetProps.HasProp(wep, "m_hObjectBeingBuilt") )
					last_active_building = NetProps.GetPropEntity(wep, "m_hObjectBeingBuilt")
				
		// This fixes taunts and switching away from custom weapons.
		// It reparents and sets owner back to the player
				if ( DisableDrawQueue )
				{
					NetProps.SetPropInt(wep, "m_iState", 0)
					DoEntFire("!self", "RunScriptCode", "self.DisableDraw()", 0, null, DisableDrawQueue)	//using delay here purposely. Won't update b/c thinks too fast!!
					NetProps.SetPropEntity(DisableDrawQueue, "m_hWeaponAssociatedWith", null)
					
					DoEntFire("!self", "SetParent", "!activator", 0, player, wep)
					wep.SetOwner(player)
					
					DisableDrawQueue = null
				}

			// These are vars are for the next part
				asdf = null
				DisableDrawQueue = null
				
			//Checks for custom weapon and/or weapon unintended for the class.
			//Passing as a custom weapon means it updates the weapon visibility, disables the base weapon,
			// then adds it to the queue to be disabled when switched away
				if ( IS_CUSTOM(wep) )
				{
					main_viewmodel.DisableDraw()		//makes firstperson weapon invisible (as well as other's class arms from the other class)
					WEP_TAUNT_FIX = true
					
					NetProps.SetPropInt(wep, "m_iState", 0)
					wep.EnableDraw()	//Have to send EnableDraw() first before DisableDraw(). Thanks Sorse(tm)!
					DoEntFire("!self", "RunScriptCode", "self.DisableDraw()", 0.01, null, wep)	//using delay here purposely. Won't update b/c thinks too fast!!
					
					asdf = PlayerLoadoutGlobal_CustomWeaponModels_TP[player][wepC]	//reads from table
					asdf.EnableDraw()
					DoEntFire("!self", "RunScriptCode", "self.EnableDraw()", 0.01, null, asdf)	//using delay here purposely. Won't update b/c thinks too fast!!
					
					DisableDrawQueue = asdf	//custom weapon disables visibility at beginning of think script.
				}
				else if ( IS_UNINTENDED(wep) ) {
					WEP_TAUNT_FIX = true
					main_viewmodel.DisableDraw() //makes firstperson weapon invisible (as well as other's class arms from the other class)
				}
				
				for (local i = 0; i <= playerscope.Wearables.len() - 1; i++) {
					if ( playerscope.Wearables[i].IsValid() ) {
						NetProps.SetPropInt(playerscope.Wearables[i], "m_AttributeManager.m_Item.m_bInitialized", 0)
						EntFireByHandle(playerscope.Wearables[i], "RunScriptCode", "NetProps.SetPropInt(self, \x22m_AttributeManager.m_Item.m_bInitialized\x22, 1)", 0.0, playerscope.Wearables[i], playerscope.Wearables[i])
					}
				}
			// this part reads from the bonemerged arms table
			// who ever the class is, and which ever arms they are using, this will put it as a value...

				if ( player in PlayerLoadoutGlobal_BonemergedArms ) {
					local bonemerged = PlayerLoadoutGlobal_BonemergedArms[player]
					
				// if ShortCircuit, disable the class arms' visibility
					if ( wep.GetClassname() == "tf_weapon_mechanical_arm" ) { // Short Circuit
						bonemerged.DisableDraw()
					}
					else if ( IS_CUSTOM(wep) || IS_UNINTENDED(wep) ) {
						bonemerged.EnableDraw()
					}
				}
			}
			return THINK_VMFIX_DELAY
		}
		AddThinkToEnt(main_viewmodel, entscriptname)	//adds think script
	}
}


//-----------------------------------------------------------------------------
// Purpose: Finds a weapon by slot.
// function ReturnWeaponBySlotBool = slot number must be positive
// function GTFW_ReturnWeaponBySlotBool = Does the same as above but slot must be negative, used by this script 
//-----------------------------------------------------------------------------
::ReturnWeaponBySlotBool <- function(wep=null, slot=0)
{
	if ( type ( slot ) == "integer" ) {
		if ( wep.GetSlot() == slot ) {
			return true
		}
	}
	return false
}
::GTFW_ReturnWeaponBySlotBool <- function(wep=null, slot=-0.0)
{
	if ( type ( slot ) == "integer" || slot == -0.0 ) {
		local asdf = split(slot.tostring()+"a",abs(slot).tostring())[0]
		if ( asdf == "-" )	//if slot checks as negative... test!
		{
			slot = abs(slot)
			if ( wep.GetSlot() == slot )
			{
				return true
			}
		}
	}
	return false
}


//-----------------------------------------------------------------------------
// Purpose: Creates tf_wearable or tf_wearable_vm.
//	-> See top of script for more info
//-----------------------------------------------------------------------------
::CTFPlayer.GTFW_CreateCustomWeaponModel <- function(wearabletype=false, baseitem=null, weapon_model=null)
{
//Error if param wearabletype isn't bool
	if ( type( wearabletype ) != "bool" ) {
		GTFW.DevPrint("GTFW_CreateCustomWeaponModel", "Function failed. Parameter 1 is not a bool. Returning null.")
		return null
	}
	
//for "targetname" keyvalue later
	local modelname = ["","",""]
	if ( type( weapon_model ) == "string" ) modelname = split(weapon_model, "/.")

//Precaching and updating models if Engineer has the Gunslinger	
	local main_viewmodel = NetProps.GetPropEntity(this, "m_hViewModel")

	if ( type( weapon_model ) == "string" )
	{
		PrecacheModel(weapon_model)
		weapon_model = GetModelIndex(weapon_model)
	}
	if ( this.HasGunslinger() && baseitem == main_viewmodel ) {
		PrecacheModel(GTFW_MODEL_ARMS[10])
		weapon_model = GetModelIndex(GTFW_MODEL_ARMS[10])
	}
	
	local wearable_handle = null
	
// bool. If true, make a tf_wearable. If false, tf_wearable_vm.
	if (wearabletype)
	{
		wearable_handle = Entities.CreateByClassname("tf_wearable")
	}
	else
	{
		wearable_handle = Entities.CreateByClassname("tf_wearable_vm")
	}
	
	
// our properties. Taken from source code for Super Zombie Fortress + SCP Secret Fortress
	wearable_handle.SetAbsOrigin(this.GetLocalOrigin())
	wearable_handle.SetAbsAngles(this.GetLocalAngles())
	NetProps.SetPropInt(wearable_handle, "m_bValidatedAttachedEntity", 1)
	NetProps.SetPropInt(wearable_handle, "m_iTeamNum", this.GetTeam())
	NetProps.SetPropInt(wearable_handle, "m_Collision.m_usSolidFlags", Constants.FSolid.FSOLID_NOT_SOLID)
	NetProps.SetPropInt(wearable_handle, "m_CollisionGroup", 11)
	NetProps.SetPropInt(wearable_handle, "m_fEffects", 129)
	
//	NetProps.SetPropInt(wearable_handle, "m_AttributeManager.m_Item.m_iEntityQuality", 0)
	NetProps.SetPropInt(wearable_handle, "m_AttributeManager.m_Item.m_iEntityLevel", 1)
	//NetProps.SetPropInt(wearable_handle, "m_AttributeManager.m_Item.m_bInitialized", 1)	//Seems to bug with upgrades stations / MvM
	NetProps.SetPropInt(wearable_handle, "m_AttributeManager.m_Item.m_bOnlyIterateItemViewAttributes", 1)	// Removes all pre-existing stats of the weapon.
	NetProps.SetPropInt(wearable_handle, "m_AttributeManager.m_iReapplyProvisionParity", 3)
	
	NetProps.SetPropEntity(wearable_handle, "m_hOwnerEntity", this)
	wearable_handle.SetOwner(this)
	
	NetProps.SetPropInt(wearable_handle, "m_nModelIndex", weapon_model)
	//NetProps.SetPropInt(wearable_handle, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", itemindex)
	
// for tf_wearable (non-vm)
	if ( baseitem == null)
	{
		Entities.DispatchSpawn(wearable_handle)
		DoEntFire("!self", "SetParent", "!activator", 0, this, wearable_handle)
	}
//for class arms
	else if ( baseitem == main_viewmodel)
	{
		Entities.DispatchSpawn(wearable_handle)
		DoEntFire("!self", "SetParent", "!activator", 0, main_viewmodel, wearable_handle)
		this.EquipWearableViewModel(wearable_handle)
	}
// for viewmodel
	else if ( baseitem && wearable_handle.GetClassname() == "tf_wearable_vm" )
	{
		NetProps.SetPropEntity(wearable_handle, "m_hWeaponAssociatedWith", baseitem)
		NetProps.SetPropEntity(baseitem, "m_hExtraWearableViewModel", wearable_handle)
		Entities.DispatchSpawn(wearable_handle)
		DoEntFire("!self", "SetParent", "!activator", 0, main_viewmodel, wearable_handle)
		this.EquipWearableViewModel(wearable_handle)
	}
//for world model (weapons only)
	else if ( baseitem && wearable_handle.GetClassname() == "tf_wearable" )
	{
	//	NetProps.SetPropEntity(wearable_handle, "m_hWeaponAssociatedWith", baseitem)
		NetProps.SetPropEntity(baseitem, "m_hExtraWearable", wearable_handle)
		Entities.DispatchSpawn(wearable_handle)
		DoEntFire("!self", "SetParent", "!activator", 0, this, wearable_handle)
		wearable_handle.DisableDraw()
		
	//	DoEntFire("!self", "AddOutput", "rendermode 1", 0, this, wearable_handle)
	//	DoEntFire("!self", "Color", "0 0 255", 0, this, wearable_handle)//colored weapons!
	//	DoEntFire("!self", "Clearparent", "", 0, null, baseitem)	//Disables the baseitem from appearing in thirdperson. However, various bugs like weapon appearing while taunting, or firing effecting origin messing up...
	}
	
// we place this in an array, for easy finding and clean-up!
	if ( wearable_handle ) {
		if( this.ValidateScriptScope() )
		{
			local scope = this.GetScriptScope()
			scope.Weapons.append(wearable_handle)
		}
	}
	
	wearable_handle.KeyValueFromString("targetname",format("tf_wearable_vscript_GTFW_%d_%d_%s",this.entindex(),GetItemID(wearable_handle),modelname[modelname.len()-2]))

	return wearable_handle
}



//-----------------------------------------------------------------------------
// The following are the PUBLIC FUNCTIONS
//-----------------------------------------------------------------------------



//-----------------------------------------------------------------------------
// Purpose: Shows in console all equips on host of listen server.
//	-> See top of script for more info
//-----------------------------------------------------------------------------

::CheckItems <- function()
{
	local ActiveWeapon = GetListenServerHost().GetActiveWeapon()
	local ActiveWeaponID = GetItemID(ActiveWeapon)

	Say(GetListenServerHost(), " ", false)
	Say(GetListenServerHost(), format("Active Slot%i [%s] (ItemID = %i)", ActiveWeapon.GetSlot(), ActiveWeapon.GetClassname(), ActiveWeaponID), false)
	
	for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
	{
		local wep = NetProps.GetPropEntityArray(GetListenServerHost(), "m_hMyWeapons", i)
		local wep_itemID = GetItemID(wep)
		ClientPrint(GetListenServerHost(), 2, i+" "+wep+" (ItemID = "+wep_itemID+")" )
		
		if ( wep && wep != ActiveWeapon)
		{
			Say(GetListenServerHost(), format("Slot%i [%s] (ItemID = %i)", wep.GetSlot(), wep.GetClassname(), wep_itemID), false)
		}
	}
}

::CheckAttrItems <- function()
{
	printl(NetProps.GetPropInt(GetListenServerHost().GetActiveWeapon(), "m_AttributeManager.m_Item.m_AttributeList.m_Attributes.lengthproxy.lengthprop20"))
	for (local i = 0; i < 20; i++)
	{
		local z = ""
		i < 10 ? z = 0 : z = "";
		local wep = NetProps.GetPropInt(GetListenServerHost().GetActiveWeapon(), "m_AttributeManager.m_Item.m_AttributeList.m_Attributes.0"+z+i+".m_iAttributeDefinitionIndex")
		local wep_val = NetProps.GetPropInt(GetListenServerHost().GetActiveWeapon(), "m_AttributeManager.m_Item.m_AttributeList.m_Attributes.0"+z+i+".m_iRawValue32")
		local currency = NetProps.GetPropInt(GetListenServerHost().GetActiveWeapon(), "m_AttributeManager.m_Item.m_AttributeList.m_Attributes.0"+z+i+".m_nRefundableCurrency")
		printl(format("%i %i %i %i",i,wep,wep_val,currency) )
	}
}
::CheckAttrPlayer <- function()
{
	printl(NetProps.GetPropInt(GetListenServerHost(), "m_AttributeList.m_Attributes.lengthproxy.lengthprop20"))
	for (local i = 0; i < 20; i++)
	{
		local z = ""
		i < 10 ? z = 0 : z = "";
		local wep = NetProps.GetPropInt(GetListenServerHost(), "m_AttributeList.m_Attributes.0"+z+i+".m_iAttributeDefinitionIndex")
		local wep_val = NetProps.GetPropInt(GetListenServerHost(), "m_AttributeList.m_Attributes.0"+z+i+".m_iRawValue32")
		local currency = NetProps.GetPropInt(GetListenServerHost(), "m_AttributeList.m_Attributes.0"+z+i+".m_nRefundableCurrency")
		printl(format("%i %i %i %i",i,wep,wep_val,currency) )
	}
}

::SetAttrPlayer <- function()
{
	printl(NetProps.GetPropInt(GetListenServerHost(), "m_AttributeList.m_Attributes.lengthproxy.lengthprop20"))
	for (local i = 0; i < 20; i++)
	{
		local z = ""
		i < 10 ? z = 0 : z = "";
		NetProps.SetPropInt(GetListenServerHost().GetActiveWeapon(), "m_AttributeManager.m_Item.m_NetworkedDynamicAttributesForDemos.m_Attributes.0"+z+i+".m_iAttributeDefinitionIndex", 1)
	}
}


//-----------------------------------------------------------------------------
//Purpose: Finds an empty weapon equipment slot and gives weapon to the player
//	-> See top of script for more info
//-----------------------------------------------------------------------------

::CTFPlayer.GiveWeapon <- function(weapon=null,force_overwrite=null,force_clear_stats=null,force_custom=null,force_stats_function=null,force_class_arms=null,force_custom_w_model=null,force_custom_v_model=null,force_extra_wearable=null,force_ammotype=null,force_clipsize=null,force_ammoreserve=null)
{
	if ( weapon == null ) {
		GTFW.DevPrint("GiveWeapon", "Function failed. Parameter is null. Returning null.")
		return null
	}
	
	local GivenWeapon = null
// Searches for the correct item based on parameter 'weapon'...
	local baseitem = this.ReturnWeaponTable(weapon)
	if ( baseitem == null ) {
		GTFW.DevPrint("GiveWeapon", "Function failed at finding item \x22"+weapon+"\x22. Returning null.")
		return null
	}
	
	local NewWeapon = baseitem.className
	local CTFClass = baseitem.tf_class
	local ItemID = baseitem.itemID
	local Slot = baseitem.slot
	local AmmoType = baseitem.ammo_type
	local ClipSize = baseitem.clip
	local AmmoReserve = baseitem.reserve
	local Extra_Wearable = baseitem.wearable
	
	local w_model = baseitem.w_model
	local v_model = baseitem.v_model
	local stats_function = baseitem.func
	local class_arms = baseitem.class_arms
	
	//local gtfw_id = baseitem.gtfw_id
	
	if( this.ValidateScriptScope() )
	{
		local scope = this.GetScriptScope()
		if ( !("Weapons" in scope) ) {
			scope.Weapons <- []
		}
		if ( !("Wearables" in scope) ) {
			scope.Wearables <- []
		}
	}
	
// Purpose: MvM / upgrading weapons via upgrade station
// If trying to find a weapon's slot (by "weapon = TF_WEAPONSLOTS.PRIMARY", for example), replace the weapon's ID based on what ever the weapon you are currently using.
	if ( ItemID < 0 )	//searching by negative numbers designates weapon slots (-0.0, -1, up to -6)
		ItemID = GetItemID(this.ReturnWeaponBySlot(Slot))
	
//Forced overrides parameters...
// These are already defaulted to null if not filled out 
	if ( type( force_ammotype ) == "integer" )
		AmmoType = force_ammotype
	
	if ( type( force_clipsize ) == "integer" )
		ClipSize = force_clipsize
	
	if ( type( force_ammoreserve ) == "integer" )
		AmmoReserve = force_ammoreserve
	
	if ( type( force_extra_wearable ) == "string" || type( force_extra_wearable ) == "integer" )
		Extra_Wearable = force_extra_wearable
	
	if ( force_stats_function )
		stats_function = force_stats_function
	
	
	if ( type( force_custom_v_model ) == "string" && force_custom_w_model == null && force_custom_v_model) {
		w_model = force_custom_v_model
	}
	else if ( type( force_custom_w_model ) == "string" && force_custom_w_model && force_custom_v_model == null) {
		v_model = force_custom_w_model
	}
	
// Purpose: searches inventory, and adds our weapon over m_hMyWeapons slot
// First it finds the old weapon to delete, based on slot.
// If there is no weapon to delete, resets itself to find a `null` slot and places new weapon there.

	//Forced overrides...
	if ( type( force_overwrite ) == "bool" ) {
		if ( force_overwrite == false ) {
			CVAR_GTFW_GIVEWEAPON_UPDATE_WEP_IDX_ONLY = true 
		}
		else if ( force_overwrite ) {
			CVAR_GTFW_GIVEWEAPON_UPDATE_WEP_IDX_ONLY = false 
		}
	}
	
	if ( CVAR_GTFW_GIVEWEAPON_UPDATE_WEP_IDX_ONLY == false ) {
		for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++) {
			
			local wep = NetProps.GetPropEntityArray(this, "m_hMyWeapons", i)
			
			if ( wep == null ) continue
				if ( CVAR_GTFW_GIVEWEAPON_REPLACE_WEAPONS && wep.GetSlot() == Slot ) {
					NetProps.SetPropEntityArray(this, "m_hMyWeapons", null, i)
					KillWepAll(wep)
					break
				}
		}
		for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++) {
			
			local wep = NetProps.GetPropEntityArray(this, "m_hMyWeapons", i)
			
			if ( wep ) continue
				GivenWeapon = this.AddWeapon(NewWeapon, ItemID, i)
				break
		}
		
	// Purpose: seems to throw an error only if it didn't create a weapon,
	// that can only happen if weps in `m_hMyWeapons` exceeds GLOBAL_WEAPON_COUNT
		if ( GivenWeapon == null ) {
			GTFW.DevPrint("GiveWeapon", "Function failed at giving weapon \x22"+weapon+"\x22. Returning null.")
			return null
		}
	}
	else {
		GivenWeapon = this.ReturnWeaponBySlot(Slot)
		if ( ItemID >= 0 ) {
			NetProps.SetPropInt(GivenWeapon, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", ItemID )
		}
		else {
			NetProps.SetPropInt(GivenWeapon, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", GetItemID(GivenWeapon) )
		}
		
		if ( GivenWeapon == null ) {
			GTFW.DevPrint("GiveWeapon", "Function failed at finding replacement weapon \x22"+weapon+"\x22. Returning null.")
			return null
		}
	}
	
// Purpose: Adds custom model, updates arms, runs custom stats function, if any apply.
// Updates weapon to become custom if model, function or class_arms is found in `baseitem` table.

	if ( (ItemID & (1<<20)) || w_model || v_model || stats_function || class_arms || force_custom || force_clear_stats || force_stats_function || force_class_arms )	// force_custom stuff added here
	{
	// Set our weapon to custom
		IS_CUSTOM(GivenWeapon, true)	//marks our weapon as custom (which means "ItemDefinitionIndex" bit 20 set, and "ItemIDHigh" bit 6 set)
		
	 // Modifies ammo if ammo types don't match
		if ( AmmoType != GivenWeapon.GetPrimaryAmmoType() ) {
			NetProps.SetPropInt(GivenWeapon, "LocalWeaponData.m_iPrimaryAmmoType", AmmoType)
		}
	// If our weapon does not use original stats (use_original_stats=false), clear them.
		if ( baseitem.clear_stats == false || force_clear_stats == true ) {
			NetProps.SetPropInt(GivenWeapon, "m_AttributeManager.m_Item.m_bOnlyIterateItemViewAttributes", 1)	// Removes all pre-existing stats of the weapon.
		}
	// Runs custom weapon function if it exists
		if ( stats_function ) {
			stats_function(GivenWeapon, this)
			GivenWeapon.ReapplyProvision()
		}
	// Updates our arms IF FORCED...
		if ( type(force_class_arms) == "string" && force_class_arms ) {
			class_arms = force_class_arms
		}
	}
	
// Purpose: Extra wearable ( if applicable )
	if ( Extra_Wearable )
		this.CreateCustomWearable(Extra_Wearable, GivenWeapon)	// creates a tf_wearable and associates it with a weapon

	
// Purpose: Updates models	
	if ( w_model == null && v_model == null && force_custom_w_model == null && force_custom_v_model == null ) {
		w_model = NetProps.GetPropInt(GivenWeapon, "m_iWorldModelIndex")
		v_model = NetProps.GetPropInt(GivenWeapon, "m_iWorldModelIndex")
	}
	else if ( w_model == null && v_model) {
		w_model = v_model
	}
	else if ( w_model && v_model == null) {
		v_model = w_model
	}


// Purpose: Finalize weapon.
// Adds AmmoFix wearable (to fix ammo), update arms/viewmodels, updates world weapon models, updates viewmodel's Think script
	this.UpdateAmmoFix(GivenWeapon,CTFClass,AmmoType,ClipSize,AmmoReserve)
	this.UpdateArms(GivenWeapon, class_arms)
	this.UpdateWeapon(GivenWeapon, w_model, v_model)
	this.AddThinkToViewModel()
	
	
// Purpose: Switches to another weapon if active one was deleted
	if (CVAR_GTFW_GIVEWEAPON_AUTO_SWITCH)
	{
		this.SwitchToActive(GivenWeapon)
	}
	
	if ( type( weapon ) == "integer" )
		weapon = baseitem.name
	GTFW.DevPrint("GiveWeapon", "Gave "+GTFW.GetNetName(this)+" item \x22"+weapon+"\x22!")
	return GivenWeapon
}


//-----------------------------------------------------------------------------
//Purpose: Deletes a weapon equipped on player
//	-> See top of script for more info
// Accepts handles, entity classnames, strings, slots (use negatives)
//-----------------------------------------------------------------------------

::CTFPlayer.DeleteWeapon <- function(weapon=null)
{
	if ( weapon == null ) {
		GTFW.DevPrint("DeleteWeapon", "Function failed. Parameter is null. Returning false.")
		return false
	}

//converts our weapon from string/ID to an itemID...
	local baseitem = this.ReturnWeaponTable(weapon)
	if ( baseitem == null ) {
		GTFW.DevPrint("DeleteWeapon", "Function failed. Could not find \x22"+weapon+"\x22. Returning null.")
		return false
	}
	local DeleteThis = baseitem.itemID
	local Slot = baseitem.slot
	local wep = null
	
	for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
	{
		local wep = NetProps.GetPropEntityArray(this, "m_hMyWeapons", i)
		
		if ( wep )
		{
			if ( GTFW_ReturnWeaponBySlotBool(wep, weapon) )
			{
				DeleteThis = GetItemID(wep)
			}
			if ( GetItemID(wep) == DeleteThis )
			{
				KillWepAll(wep)
				break
			}
		}
	}
	
// switches to another weapon if active one was deleted
	if (CVAR_GTFW_DELETEWEAPON_AUTO_SWITCH) {
		if ( wep == null ) {
			this.SwitchToActive(null)
		}
	}
	
	return true
}


//-----------------------------------------------------------------------------
// Purpose: Returns weapon, searched by slot. 
// Requested by Lizard of Oz
//-----------------------------------------------------------------------------
::CTFPlayer.ReturnWeaponBySlot <- function(slot)
{
	if ( type( slot ) != "integer" ) {
		GTFW.DevPrint("ReturnWeaponBySlot", "Parameter 1 not an integer. Returning null.")
		return null
	}
	local GetThis = null
	local YourGunFoundBySaxtonHale = null

	for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
	{
		local wep = NetProps.GetPropEntityArray(this, "m_hMyWeapons", i)
		
		if ( wep )
		{
			if ( ReturnWeaponBySlotBool(wep, slot) )
			{
				GetThis = wep.GetClassname()
			}
			if ( wep.GetClassname() == GetThis )
			{
				YourGunFoundBySaxtonHale = wep
				break
			}
		}
	}
	return YourGunFoundBySaxtonHale
}


//-----------------------------------------------------------------------------
// Purpose: disables being able to switch to a weapon.
//	-> See top of script for more info
//-----------------------------------------------------------------------------

::CTFPlayer.DisableWeapon <- function(weapon=null)
{
//Error if param is null
	if ( weapon == null ) {
		GTFW.DevPrint("DisableWeapon", "Function failed. Parameter is null. Returning null.")
		return null
	}
//searches for the correct item based on parameter 'weapon'...
	local baseitem = this.ReturnWeaponTable(weapon)
	if ( baseitem == null ) {
		GTFW.DevPrint("DisableWeapon", "Function failed. Could not find \x22"+weapon+"\x22. Returning null.")
		return null
	}
	
	local DisableThis = baseitem.itemID
	
	local BrokenGun = null
	local wep = null
	
	for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
	{
		local wep = NetProps.GetPropEntityArray(this, "m_hMyWeapons", i)
		
		if ( wep )
		{
			if ( GTFW_ReturnWeaponBySlotBool(wep, weapon) )
			{
				DisableThis = GetItemID(wep)
			}
			if ( GetItemID(wep) == DisableThis )
			{
				local AmmoFix = format("tf_wearable_vscript_%d_%d_%s",this.entindex(),65535,"AMMOFIX")
				AmmoFix = Entities.FindByName(null, AmmoFix.tostring());
				if ( !AmmoFix ) {
					AmmoFix = this.CreateCustomWearable("models/empty.mdl")
				}
				
				if ( wep.GetPrimaryAmmoType() == TF_AMMO.PRIMARY ) {
					AmmoFix.AddAttribute("maxammo primary reduced", 0, -1)
					AmmoFix.ReapplyProvision()
					EntityOutputs.AddOutput(AmmoFix, "OnUser1", "", "", "", 0.0, -1)	//used to flag for Primary Ammo
				}
				else if ( wep.GetPrimaryAmmoType() == TF_AMMO.SECONDARY ) {
					AmmoFix.AddAttribute("maxammo secondary reduced", 0, -1)
					AmmoFix.ReapplyProvision()
					EntityOutputs.AddOutput(AmmoFix, "OnUser2", "", "", "", 0.0, -1)	//used to flag for Secondary Ammo
				}
				else if ( wep.GetPrimaryAmmoType() == TF_AMMO.GRENADES1 ) {
					AmmoFix.AddAttribute("maxammo grenades1 increased", 0, -1)
					AmmoFix.ReapplyProvision()
					EntityOutputs.AddOutput(AmmoFix, "OnUser4", "", "", "", 0.0, -1)	//used to flag for GRENADES1 Ammo
				}
				wep.SetClip1(0)
				NetProps.SetPropIntArray(this, "m_iAmmo", 0, wep.GetPrimaryAmmoType())
				
				BrokenGun = wep
				break
			}
		}
	}
	
// switches to another weapon if active one was disabled
	if (CVAR_GTFW_DISABLEWEAPON_AUTO_SWITCH) {
		this.SwitchToActive(null)
	}
	
	return BrokenGun
}


//-----------------------------------------------------------------------------
//Purpose: Enables weapon if it was disabled.
//	-> See top of script for more info
//-----------------------------------------------------------------------------
::CTFPlayer.EnableWeapon <- function(weapon=null)
{
//Error if param is null
	if ( weapon == null ) {
		GTFW.DevPrint("EnableWeapon", "Function failed. Parameter is null. Returning null.")
		return null
	}
	local FixThis = null
//searches for the correct item based on parameter 'weapon'...
	local baseitem = this.ReturnWeaponTable(weapon)
	if ( baseitem == null ) {
		GTFW.DevPrint("EnableWeapon", "Function failed. Could not find \x22"+weapon+"\x22. Returning null.")
		return null
	}
	
	FixThis = baseitem.itemID
	
	local FixedGun = null
	
	for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
	{
		local wep = NetProps.GetPropEntityArray(this, "m_hMyWeapons", i)
		
		if ( wep )
		{
			if ( GTFW_ReturnWeaponBySlotBool(wep, weapon) )
			{
				FixThis = GetItemID(wep)
			}
			if ( GetItemID(wep) == FixThis )
			{
				local AmmoFix = format("tf_wearable_vscript_%d_%d_%s",this.entindex(),65535,"AMMOFIX")
				AmmoFix = Entities.FindByName(null, AmmoFix.tostring());
				if ( !AmmoFix ) {
					AmmoFix = this.CreateCustomWearable("models/empty.mdl")
				}
				
				if ( wep.GetPrimaryAmmoType() == TF_AMMO.PRIMARY ) {
					AmmoFix.AddAttribute("maxammo primary reduced", 1, -1)
					AmmoFix.ReapplyProvision()
					EntityOutputs.RemoveOutput(AmmoFix, "OnUser1", "", "", "")	//used to flag for Primary Ammo
				}
				else if ( wep.GetPrimaryAmmoType() == TF_AMMO.SECONDARY ) {
					AmmoFix.AddAttribute("maxammo secondary reduced", 1, -1)
					AmmoFix.ReapplyProvision()
					EntityOutputs.RemoveOutput(AmmoFix, "OnUser2", "", "", "")	//used to flag for Secondary Ammo
				}
				else if ( wep.GetPrimaryAmmoType() == TF_AMMO.GRENADES1 ) {
					AmmoFix.AddAttribute("maxammo grenades1 increased", 1, -1)
					AmmoFix.ReapplyProvision()
					EntityOutputs.RemoveOutput(AmmoFix, "OnUser4", "", "", "")	//used to flag for GRENADES1 Ammo
				}
				
				FixedGun = wep
				break
			}
		}
	}
	
// switches to another weapon if active one was disabled

	if (CVAR_GTFW_ENABLEWEAPON_AUTO_SWITCH) {
		this.SwitchToActive(FixedGun)
	}
	
	return FixedGun
}


//-----------------------------------------------------------------------------
// Purpose: Returns weapon from player as a hPlayer.
//	-> See top of script for more info
//-----------------------------------------------------------------------------
::CTFPlayer.ReturnWeapon <- function(searched_weapon=null)
{
//Error if param is null
	if ( searched_weapon == null ) {
		GTFW.DevPrint("ReturnWeapon", "Function failed. Parameter is null. Returning null.")
		return null
	}
	local GetThis = null
//searches for the correct item based on parameter 'searched_weapon'...
	local baseitem = this.ReturnWeaponTable(searched_weapon)
	if ( baseitem == null ) {
		GTFW.DevPrint("ReturnWeapon", "Function failed. Could not find \x22"+searched_weapon+"\x22. Returning null.")
		return null
	}
	GetThis = baseitem.itemID

	local YourGunFoundBySaxtonHale = null

	for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
	{
		local wep = NetProps.GetPropEntityArray(this, "m_hMyWeapons", i)
		
		if ( wep )
		{
			if ( GTFW_ReturnWeaponBySlotBool(wep, searched_weapon) )
			{
				GetThis = GetItemID(wep)
			}
			if ( GetItemID(wep) == GetThis )
			{
				YourGunFoundBySaxtonHale = wep
				break
			}
		}
	}
	
	if ( YourGunFoundBySaxtonHale == null ) {
		GTFW.DevPrint("ReturnWeapon", "Function failed. Could not find \x22"+searched_weapon+"\x22 (ID "+baseitem.itemID+"). Returning null.")
		return null
	}
	return YourGunFoundBySaxtonHale
}


//-----------------------------------------------------------------------------
// Purpose: Finds item ID of weapon.
//-----------------------------------------------------------------------------
::GetItemID <- function(wep)
{
//Error if param is wep is null
	if ( wep == null ) {
		GTFW.DevPrint("GetItemID", "Function failed. Parameter 1 is null. Returning null.")
		return null
	}
	else if ( type(wep) == "integer" ) {
		GTFW.DevPrint("GetItemID", "Function failed. Parameter 1 is integer. Returning null.")
		return wep
	}
	else if ( !NetProps.HasProp(wep, "m_AttributeManager.m_Item.m_iItemDefinitionIndex")) {
		GTFW.DevPrint("GetItemID", "Function failed. Ran weapon "+wep+". Parameter 1 is not a handle. Returning null.")
		return null
	}
// Finds item ID
	local ItemID = NetProps.GetPropInt(wep, "m_AttributeManager.m_Item.m_iItemDefinitionIndex")
//	ClientPrint(null,3,format("%s with %d",wep.tostring(),ItemID))
	
	if ( IS_CUSTOM(wep) ) { //if custom
		if ( !IsMannVsMachineMode() || !GameModeUsesUpgrades() )
			ItemID = ItemID | (1 << 20) //sets bit 20 for searching in ReturnWeaponTable
		NetProps.SetPropInt(wep, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", ItemID )
	}
	return ItemID
}



//-----------------------------------------------------------------------------
// Purpose: Updates the class arms model to another one.
// EXPERIMENTAL: Probably don't use this unless your ent count to skyrocket.
// Each weapon's viewmodel needs an ent (tf_wearable_vm) to make this work!
// You need to update it every time the player touches the resupply.
//-----------------------------------------------------------------------------
::CTFPlayer.CreateArmsModel <- function(arms_model=null)
{
	if ( arms_model == null || type( arms_model ) != "string" ) {
		GTFW.DevPrint("CreateArmsModel", "Function failed. Parameter is invalid.")
		return
	}
	local main_viewmodel = NetProps.GetPropEntity(this, "m_hViewModel")
	
// Table for bonemerged class arms, per player.
// If there is already a model, delete it.
	if ( this in PlayerLoadoutGlobal_BonemergedArms ) {
		local arms = PlayerLoadoutGlobal_BonemergedArms[this]
		if ( arms && arms.IsValid() ) {
			arms.Kill()
		}
		delete PlayerLoadoutGlobal_BonemergedArms[this]
	}
	
	local wearable_handle = this.GTFW_CreateCustomWeaponModel(false, main_viewmodel, arms_model)	// creates player class' arms, parents and does all that stuff

// Puts into a table "player = arms model" for easy tracking
	::PlayerLoadoutGlobal_BonemergedArms[this] <- wearable_handle
	
	for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
	{
		local wep = NetProps.GetPropEntityArray(this, "m_hMyWeapons", i)
		
		if ( wep )
		{
			IS_UNINTENDED(wep, true)
			local wep_mdl = NetProps.GetPropInt(wep, "m_iWorldModelIndex")
			this.UpdateWeapon(wep, wep_mdl, wep_mdl)
		}
	}
	this.UpdateArms()
	this.AddThinkToViewModel()
	return wearable_handle
}

/*Notes for viewmodels (UNFINISHED)
main_viewmodel (tf_viewmodel) needs to be the class arms model for bonemerging with weapon_arms. Cannot be unparented without complication. Cannot arbitrarily set "m_hWeapon" (netprop for current weapon entity seen by player).
weapon_arms (tf_wearable_vm) are the new class' hands that bonemerge with main_viewmodel.
baseitem (the weapon entity) displays the thirdperson model but the model it uses is the class arms.
When baseitem is unparented, it unparents the third person model from the player. However, doing this makes the weapon stay visible with taunts. (A think script fixes this)
baseitem's viewmodel animation sequences are taken from main_viewmodel.
baseitem's modelindex are the /class_arms/. Changing this changes what sequences are read by main_viewmodel.
baseitem netprop "m_iViewModelIndex" is viewmodel index. Cannot change arbitrarily.
baseitem netprop "m_iWorldModelIndex" is thirdperson index. Cannot change arbitrarily.

//How the weapon's VIEWMODEL changes visibility depending on when player is holding the weapon:
new_weapon_viewmodel (tf_wearable_vm) is another entity model parented to the existing viewmodel.
new_weapon_viewmodel links itself to the baseitem (w/ netprop "m_hWeaponAssociatedWith")
baseitem links itself to the new_weapon_viewmodel (w/ netprop "m_hExtraWearableViewModel")

//How the weapon's THIRDPERSON/WORLD MODEL changes visibility depending on when player is holding the weapon:
new_weapon_thirdperson (tf_wearable) is another entity model parented on the player.
new_weapon_thirdperson links itself to the baseitem (w/ netprop "m_hWeaponAssociatedWith")
baseitem links itself to the new_weapon_thirdperson (w/ netprop "m_hExtraWearable")

//viewmodel hierarchy:
main_viewmodel sequences [become]-> baseitem's sequences
main_viewmodel (tf_viewmodel) sequences are used by weapon_arms (tf_wearable_vm) for bonemerging
weapon_arms bonemerge with baseitem to complete the effect that class is holding the baseitem
baseitem's sequences MUST MATCH main_viewmodel's

//If you intend to update the weapon's animation sequences:
Can set Sequence of weapon via main_viewmodel. Example for Scout's Bat:
main_viewmodel.SetSequence(main_viewmodel.LookupSequence("b_draw") )

You can change which sequence list by changing the model itself to a different class arms model.
change the main_viewmodel using main_viewmodel.SetModelSimple() to change to new set of class arms
change the baseitem's modelindex (which are class_arms) to the animations you want to use for the new weapon
However, now that SetCustomViewModel() exists, changing the main_viewmodel isn't necessary anymore.
*/


//-----------------------------------------------------------------------------
// Purpose: Creates a tf_wearable and associates it with a weapon
//-----------------------------------------------------------------------------
::CTFPlayer.CreateCustomWearable <- function(wearable_model=null,weapon="")
{
//Error if param model isn't a string nor number
	if ( type( wearable_model ) != "string" && type( wearable_model ) != "integer" ) {
		GTFW.DevPrint("CreateCustomWearable", "Function failed. Parameter 1 is invalid. Returning null.")
		return null
	}

//for "targetname" keyvalue later
	local displayname = weapon
	if ( type( wearable_model ) == "string" ) {
		local modelname = split(wearable_model, "/.")
		if ( modelname[modelname.len()-2] == "empty" && weapon == "") displayname = "AMMOFIX"
		else if ( weapon == "" ) weapon = modelname[modelname.len()-2]
	}
	
// Precaches weapon if string
	local wearable_model_is_string = false
	if ( type( wearable_model ) == "string" )
	{
		PrecacheModel(wearable_model)
		wearable_model = GetModelIndex(wearable_model)
		wearable_model_is_string = true
	}
	
	local wearable = null
	if ( type( weapon ) == "instance" ) {
		while ( wearable = Entities.FindByClassname(wearable, "tf_wearable") )
		{
			if ( wearable && NetProps.GetPropEntity(wearable, "m_AttributeManager.m_hOuter").GetOwner() == this && NetProps.GetPropEntity(weapon, "m_hExtraWearable") == wearable ) {
				GTFW.DevPrint("CreateCustomWearable", "Deleting wearable... "+wearable.GetModelName()+" entindex# "+wearable.entindex() )
				
				wearable.Kill()
				break
			}
			else if ( !wearable )
				break
			else if ( !NetProps.GetPropEntity(wearable, "m_AttributeManager.m_hOuter").GetOwner() == this )
				break
		}
	}
	
// our properties. Taken from source code for Super Zombie Fortress + SCP Secret Fortress
	local wearable = Entities.CreateByClassname("tf_wearable")
	
	wearable.SetAbsOrigin(this.GetLocalOrigin())
	wearable.SetAbsAngles(this.GetLocalAngles())
	NetProps.SetPropInt(wearable, "m_bClientSideAnimation", 1)
	NetProps.SetPropInt(wearable, "m_bValidatedAttachedEntity", 1)
	NetProps.SetPropInt(wearable, "m_iTeamNum", this.GetTeam())
	NetProps.SetPropInt(wearable, "m_Collision.m_usSolidFlags", Constants.FSolid.FSOLID_NOT_SOLID)
	NetProps.SetPropInt(wearable, "m_CollisionGroup", 11)
	NetProps.SetPropInt(wearable, "m_fEffects", 129)
	
	if ( wearable_model_is_string ) {
		NetProps.SetPropInt(wearable, "m_nModelIndex", wearable_model)
		NetProps.SetPropInt(wearable, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", 65535)
	}
	else {
		NetProps.SetPropInt(wearable, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", wearable_model)
	}
	
	NetProps.SetPropInt(wearable, "m_AttributeManager.m_Item.m_iEntityQuality", 0)
	NetProps.SetPropInt(wearable, "m_AttributeManager.m_Item.m_iEntityLevel", 1)
	NetProps.SetPropInt(wearable, "m_AttributeManager.m_Item.m_bInitialized", 1)
	
	NetProps.SetPropEntity(wearable, "m_hOwnerEntity", this)
	wearable.SetOwner(this)
	
	Entities.DispatchSpawn(wearable)
	DoEntFire("!self", "SetParent", "!activator", 0, this, wearable)
	wearable.KeyValueFromString("targetname",format("tf_wearable_vscript_%d_%d_%s",this.entindex(),GetItemID(wearable),displayname.tostring()))
	

// Purpose: Cleanup
// Adds onto an array with all related wearables to be deleted every player loadout update
	if ( wearable ){
		if( this.ValidateScriptScope() )
		{
			local scope = this.GetScriptScope()
			scope.Wearables.append(wearable)
		}
	}
	
	if ( weapon ) 
		GTFW.DevPrint("CreateCustomWearable", "Created custom wearable for weapon \x22"+displayname+"\x22! "+GetItemID(wearable))

	return wearable
}

//-----------------------------------------------------------------------------
//	Purpose: Handles registering custom weapons.
//	This version gets rid of pesky ammo setting bugs by overwriting the ammo here instead of in the stats_function
//-----------------------------------------------------------------------------
::RegisterCustomWeapon <- function(item_name, weapon, use_original_stats=false, stats_function=null, custom_arms_model=null, custom_world_model=null, custom_view_model=null, custom_extra_wearable=null, ammotype=0, clipsize=-1, ammoreserve=-1)
{
//Error if params are invalid
	if ( type( item_name ) != "string" ) {
		GTFW.DevPrint("RegisterCustomWeapon", "Function failed. MUST use strings for custom item name. Returning null.")
		return null
	}
	
	/*
//Adding GTFW ID for multiple weapons with the same item definition index
	TF_CUSTOM_WEAPONS_IDs.append(TF_CUSTOM_WEAPONS_IDs.len())
	local CW_ID	= TF_CUSTOM_WEAPONS_IDs[TF_CUSTOM_WEAPONS_IDs.len() - 1]
	*/
	
//searches for the correct item based on parameter 'weapon'...
//'weapon' can be a handle, string name (ex "Brass Beast"), weapon item definition index (from items_game.txt), or a slot (negative value)

	local baseitem = GTFW_SearchEquipReg(weapon)
	if ( baseitem == null && type( weapon ) == "integer" || weapon == -0.0 ) {
		local slot = split(weapon.tostring()+"a",abs(weapon).tostring())[0]
		if ( slot == "-" )	//if weapon slot checks as negative... test!
		{
			weapon = abs(weapon)
			baseitem = {
				className	= "saxxy"
				tf_class	= 0
				slot		= weapon
				
				ammo_type	= ammotype
				clip		= clipsize
				reserve		= ammoreserve
				itemID		= -1
			}
				
		}
	}
		
	if ( ammotype == null || ammotype < 0 )
		ammotype = baseitem.ammo_type
	if ( clipsize == null || clipsize < 0 )
		clipsize = baseitem.clip
	if ( ammoreserve == null || ammoreserve < 0 )
		ammoreserve = baseitem.reserve
		
		
	local itemid = baseitem.itemID | (1 << 20)
	
	GTFW.DevPrint("", "Registering... "+item_name)
	::TF_CUSTOM_WEAPONS_REGISTRY[item_name] <- TF_CUSTOM_WEPS(item_name, baseitem.className, baseitem.tf_class, baseitem.slot, itemid, use_original_stats, stats_function, custom_arms_model, ammotype, clipsize, ammoreserve, custom_world_model, custom_view_model, custom_extra_wearable)
	
	GTFW.DevPrint("", "Register Success!")
}

//-----------------------------------------------------------------------------
//	Purpose: Fixes bots using this script
//-----------------------------------------------------------------------------
::CTFBot.GTFW_Cleanup <- CTFPlayer.GTFW_Cleanup
::CTFBot.SwitchToActive <- CTFPlayer.SwitchToActive
::CTFBot.HasGunslinger <- CTFPlayer.HasGunslinger
::CTFBot.ReturnWeaponTable <- CTFPlayer.ReturnWeaponTable
::CTFBot.GTFW_PostWepFix <- CTFPlayer.GTFW_PostWepFix
::CTFBot.AddWeapon <- CTFPlayer.AddWeapon
::CTFBot.UpdateAmmoFix <- CTFPlayer.UpdateAmmoFix
::CTFBot.UpdateArms <- CTFPlayer.UpdateArms
::CTFBot.UpdateWeapon <- CTFPlayer.UpdateWeapon
::CTFBot.GTFW_CreateCustomWeaponModel <- CTFPlayer.GTFW_CreateCustomWeaponModel
::CTFBot.AddThinkToViewModel <- CTFPlayer.AddThinkToViewModel
::CTFBot.GiveWeapon <- CTFPlayer.GiveWeapon
::CTFBot.DeleteWeapon <- CTFPlayer.DeleteWeapon
::CTFBot.ReturnWeaponBySlot <- CTFPlayer.ReturnWeaponBySlot
::CTFBot.DisableWeapon <- CTFPlayer.DisableWeapon
::CTFBot.EnableWeapon <- CTFPlayer.EnableWeapon
::CTFBot.ReturnWeapon <- CTFPlayer.ReturnWeapon
::CTFBot.CreateArmsModel <- CTFPlayer.CreateArmsModel
::CTFBot.CreateCustomWearable <- CTFPlayer.CreateCustomWearable
::CTFBot.AnnounceWeapon <- CTFPlayer.AnnounceWeapon
::CTFBot.LoadLoadout <- CTFPlayer.LoadLoadout
::CTFBot.SaveLoadout <- CTFPlayer.SaveLoadout
::CTFBot.DeleteLoadout <- CTFPlayer.DeleteLoadout

//-----------------------------------------------------------------------------
//	Purpose: Renames in case someone else was using these function names before
//-----------------------------------------------------------------------------

//RegisterCustomWeapon
::RegisterCustomWeaponEx <- RegisterCustomWeapon

//GetWeapon
::CTFPlayer.GetWeapon <- CTFPlayer.ReturnWeapon
::CTFBot.GetWeapon <- CTFPlayer.ReturnWeapon

//GetWeaponTable
::ReturnWeaponTable <- GTFW_SearchEquipReg
::GetWeaponTable <- GTFW_SearchEquipReg
::CTFPlayer.GetWeaponTable <- CTFPlayer.ReturnWeaponTable
::CTFBot.GetWeaponTable <- CTFPlayer.ReturnWeaponTable

//GetWeaponSlot
GetWeaponSlot <- ReturnWeaponSlot

//Persist Loadout
::CTFBot.GTFW_LoadLoadout <- CTFPlayer.LoadLoadout
::CTFBot.GTFW_SaveLoadout <- CTFPlayer.SaveLoadout
::CTFBot.GTFW_DeleteLoadout <- CTFPlayer.DeleteLoadout
::CTFPlayer.GTFW_LoadLoadout <- CTFPlayer.LoadLoadout
::CTFPlayer.GTFW_SaveLoadout <- CTFPlayer.SaveLoadout
::CTFPlayer.GTFW_DeleteLoadout <- CTFPlayer.DeleteLoadout